# (C) Copyright 2019-2020 Hewlett Packard Enterprise Development LP.
# Apache License 2.0

from pyaoscx import interface, common_ops
from pyaoscx import common_ops
from pyaoscx import port
from pyaoscx import mac

import json
import random
import logging


def get_vlan(vlan_id, depth=0, selector="configuration", **kwargs):
    """
    Perform a GET call to retrieve data for a VLAN table entry

    :param depth: Integer deciding how many levels into the API JSON that references will be returned.
    :param selector: Alphanumeric option to select specific information to return.  The options are 'configuration',
        'status', or 'statistics'.  If running v10.04 or later, an additional option 'writable' is included.
    :param vlan_id: Numeric ID of VLAN
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Dictionary containing port data
    """
    if kwargs["url"].endswith("/v1/"):
        return _get_vlan_v1(vlan_id, depth, selector, **kwargs)
    else:   # Updated else for when version is v10.04
        return _get_vlan(vlan_id, depth, selector, **kwargs)


def _get_vlan_v1(vlan_id, depth=0, selector="configuration", **kwargs):
    """
    Perform a GET call to retrieve data for a VLAN table entry

    :param depth: Integer deciding how many levels into the API JSON that references will be returned.
    :param selector: Alphanumeric option to select specific information to return.  The options are 'configuration',
        'status', or 'statistics'.  If running v10.04 or later, an additional option 'writable' is included.
    :param vlan_id: Numeric ID of VLAN
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Dictionary containing port data
    """
    if selector not in ['configuration', 'status', 'statistics', None]:
        raise Exception("ERROR: Selector should be 'configuration', 'status', or 'statistics'")

    target_url = kwargs["url"] + "system/vlans/%d" % vlan_id

    payload = {
        "depth": depth,
        "selector": selector
    }
    response = kwargs["s"].get(target_url, verify=False, params=payload, timeout=2)

    if not common_ops._response_ok(response, "GET"):
        logging.warning("FAIL: Getting VLAN ID '%d' table entry failed with status code %d: %s"
                        % (vlan_id, response.status_code, response.text))
        output = {}
    else:
        logging.info("SUCCESS: Getting VLAN ID '%d' table entry succeeded" % vlan_id)
        output = response.json()

    return output


def _get_vlan(vlan_id, depth=1, selector="writable", **kwargs):
    """
    Perform a GET call to retrieve data for a VLAN table entry

    :param depth: Integer deciding how many levels into the API JSON that references will be returned.
    :param selector: Alphanumeric option to select specific information to return.  The options are 'configuration',
        'status', or 'statistics'.  If running v10.04 or later, an additional option 'writable' is included.
    :param vlan_id: Numeric ID of VLAN
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Dictionary containing port data
    """
    if selector not in ['configuration', 'status', 'statistics', 'writable', None]:
        raise Exception("ERROR: Selector should be 'configuration', 'status', 'statistics', or 'writable'")

    target_url = kwargs["url"] + "system/vlans/%d" % vlan_id
    payload = {
        "depth": depth,
        "selector": selector
    }
    response = kwargs["s"].get(target_url, verify=False, params=payload, timeout=2)
    if not common_ops._response_ok(response, "GET"):
        logging.warning("FAIL: Getting VLAN ID '%d' table entry failed with status code %d: %s"
                        % (vlan_id, response.status_code, response.text))
        output = {}
    else:
        logging.info("SUCCESS: Getting VLAN ID '%d' table entry succeeded" % vlan_id)
        output = response.json()

    return output


def vlan_get_all_mac_info(vlan_id, **kwargs):
    """
    Perform GET calls to get info for all MAC address(es) of VLAN

    :param vlan_id: Numeric ID of VLAN
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: List of dictionaries containing MAC info
    """

    mac_uris_percents = mac.get_all_mac_addrs(vlan_id, **kwargs)

    mac_data_list = []
    for mac_uri_percent in mac_uris_percents:
        uri_split = mac_uri_percent.split('/')
        mac_addr = common_ops._replace_percents(uri_split[-1])
        mac_type = uri_split[-2]
        vlan_id = int(uri_split[-4])

        mac_data = mac.get_mac_info(vlan_id, mac_type, mac_addr, **kwargs)
        mac_data_list.append(mac_data)

    return mac_data_list


def get_all_vlans(**kwargs):
    """
    Perform a GET call to get a list (or dictionary)  of all entries in VLANs table

    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: List/dict of all VLANs in the table
    """
    target_url = kwargs["url"] + "system/vlans"

    response = kwargs["s"].get(target_url, verify=False)

    if not common_ops._response_ok(response, "GET"):
        logging.warning("FAIL: Getting list/dict of all VLAN table entries failed with status code %d: %s"
              % (response.status_code, response.text))
    else:
        logging.info("SUCCESS: Getting list/dict of all VLAN table entries succeeded")

    vlans = response.json()
    return vlans


def create_vlan(vlan_id, vlan_name, vlan_desc=None, vlan_type="static", admin_conf_state="up", **kwargs):
    """
    Perform a POST call to create a new VLAN.

    :param vlan_id: Numeric ID of VLAN
    :param vlan_name: Alphanumeric name of VLAN
    :param vlan_desc: Optional description to add to VLAN
    :param vlan_type: VLAN type. Defaults to "static" if not specified
    :param admin_conf_state: Optional administratively-configured state of VLAN.
        Only configurable for static VLANs. Defaults to "up" for static VLANs.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        return _create_vlan_v1(vlan_id, vlan_name, vlan_desc, vlan_type, admin_conf_state, **kwargs)
    else:  # Updated else for when version is v10.04
        return _create_vlan(vlan_id, vlan_name, vlan_desc, vlan_type, admin_conf_state, **kwargs)


def _create_vlan_v1(vlan_id, vlan_name, vlan_desc=None, vlan_type="static", admin_conf_state="up", **kwargs):
    """
    Perform a POST call to create a new VLAN.

    :param vlan_id: Numeric ID of VLAN
    :param vlan_name: Alphanumeric name of VLAN
    :param vlan_desc: Optional description to add to VLAN
    :param vlan_type: VLAN type. Defaults to "static" if not specified
    :param admin_conf_state: Optional administratively-configured state of VLAN.
        Only configurable for static VLANs. Defaults to "up" for static VLANs.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    vlans_list = get_all_vlans(**kwargs)

    if "/rest/v1/system/vlans/%s" % vlan_id not in vlans_list:
        vlan_data = {"id": vlan_id, "name": vlan_name, "type": vlan_type}

        if vlan_desc is not None:
            vlan_data["description"] = vlan_desc

        if vlan_type == "static":
            # admin-configured state can only be set on static VLANs
            vlan_data["admin"] = admin_conf_state

        target_url = kwargs["url"] + "system/vlans"
        post_data = json.dumps(vlan_data, sort_keys=True, indent=4)

        response = kwargs["s"].post(target_url, data=post_data, verify=False)

        if not common_ops._response_ok(response, "POST"):
            logging.warning("FAIL: Adding VLAN table entry '%s' failed with status code %d: %s"
                  % (vlan_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Adding VLAN table entry '%s' succeeded" % vlan_name)
            return True
    else:
        logging.info("SUCCESS: No need to create VLAN ID '%d' since it already exists" % vlan_id)
        return True


def _create_vlan(vlan_id, vlan_name, vlan_desc=None, vlan_type="static", admin_conf_state="up", **kwargs):
    """
    Perform a POST call to create a new VLAN.

    :param vlan_id: Numeric ID of VLAN
    :param vlan_name: Alphanumeric name of VLAN
    :param vlan_desc: Optional description to add to VLAN
    :param vlan_type: VLAN type. Defaults to "static" if not specified
    :param admin_conf_state: Optional administratively-configured state of VLAN.
        Only configurable for static VLANs. Defaults to "up" for static VLANs.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    vlans_dict = get_all_vlans(**kwargs)

    if str(vlan_id) not in vlans_dict:
        vlan_data = {"id": vlan_id, "name": vlan_name, "type": vlan_type}

        if vlan_desc is not None:
            vlan_data["description"] = vlan_desc

        if vlan_type == "static":
            # admin-configured state can only be set on static VLANs
            vlan_data["admin"] = admin_conf_state

        target_url = kwargs["url"] + "system/vlans"
        post_data = json.dumps(vlan_data, sort_keys=True, indent=4)

        response = kwargs["s"].post(target_url, data=post_data, verify=False)

        if not common_ops._response_ok(response, "POST"):
            logging.warning("FAIL: Adding VLAN table entry '%s' failed with status code %d: %s"
                  % (vlan_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Adding VLAN table entry '%s' succeeded" % vlan_name)
            return True
    else:
        logging.info("SUCCESS: No need to create VLAN ID '%d' since it already exists" % vlan_id)
        return True


def modify_vlan(vlan_id, vlan_name=None, vlan_desc=None, admin_conf_state=None, **kwargs):
    """
    Perform GET and PUT calls to modify an existing VLAN.

    :param vlan_id: Numeric ID of VLAN
    :param vlan_name: Optional Alphanumeric name of VLAN. Won't be modified if not specified.
    :param vlan_desc: Optional description to add to VLAN. Won't be modified if not specified.
    :param admin_conf_state: Optional administratively-configured state of VLAN. Won't be modified if not specified.
        Only configurable for static VLANs.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        return _modify_vlan_v1(vlan_id, vlan_name, vlan_desc, admin_conf_state, **kwargs)
    else:  # Updated else for when version is v10.04
        return _modify_vlan(vlan_id, vlan_name, vlan_desc, admin_conf_state, **kwargs)


def _modify_vlan_v1(vlan_id, vlan_name=None, vlan_desc=None, admin_conf_state=None, **kwargs):
    """
    Perform GET and PUT calls to modify an existing VLAN.

    :param vlan_id: Numeric ID of VLAN
    :param vlan_name: Optional Alphanumeric name of VLAN. Won't be modified if not specified.
    :param vlan_desc: Optional description to add to VLAN. Won't be modified if not specified.
    :param admin_conf_state: Optional administratively-configured state of VLAN. Won't be modified if not specified.
        Only configurable for static VLANs.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """

    vlan_data = get_vlan(vlan_id, depth=0, selector="configuration",  **kwargs)
    vlan_data.pop('id', None)  # id cannot be modified

    if vlan_name is not None:
        vlan_data["name"] = vlan_name

    if vlan_desc is not None:
        vlan_data["description"] = vlan_desc

    if vlan_data['type'] == "static" and admin_conf_state is not None:
        # admin-configured state can only be set on static VLANs
        vlan_data["admin"] = admin_conf_state

    target_url = kwargs["url"] + "system/vlans/%d" % vlan_id
    put_data = json.dumps(vlan_data, sort_keys=True, indent=4)

    response = kwargs["s"].put(target_url, data=put_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Modifying VLAN ID '%d' failed with status code %d: %s"
                        % (vlan_id, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Modifying VLAN ID '%d' succeeded" % vlan_id)
        return True


def _modify_vlan(vlan_id, vlan_name=None, vlan_desc=None, admin_conf_state=None, **kwargs):
    """
    Perform GET and PUT calls to modify an existing VLAN.

    :param vlan_id: Numeric ID of VLAN
    :param vlan_name: Optional Alphanumeric name of VLAN. Won't be modified if not specified.
    :param vlan_desc: Optional description to add to VLAN. Won't be modified if not specified.
    :param admin_conf_state: Optional administratively-configured state of VLAN. Won't be modified if not specified.
        Only configurable for static VLANs.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """

    vlan_data = get_vlan(vlan_id, depth=1, selector="writable",  **kwargs)

    if vlan_name is not None:
        vlan_data["name"] = vlan_name

    if vlan_desc is not None:
        vlan_data["description"] = vlan_desc

    if vlan_data['type'] == "static" and admin_conf_state is not None:
        # admin-configured state can only be set on static VLANs
        vlan_data["admin"] = admin_conf_state

    target_url = kwargs["url"] + "system/vlans/%d" % vlan_id
    put_data = json.dumps(vlan_data, sort_keys=True, indent=4)

    response = kwargs["s"].put(target_url, data=put_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Modifying VLAN ID '%d' failed with status code %d: %s"
                        % (vlan_id, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Modifying VLAN ID '%d' succeeded" % vlan_id)
        return True


def create_vlan_and_svi(vlan_id, vlan_name, vlan_port_name, vlan_int_name, vlan_desc=None, ipv4=None,
                        vrf_name="default", vlan_port_desc=None, **kwargs):
    """
    Perform POST and PUT calls to create a new VLAN and SVI.

    :param vlan_id: Numeric ID of VLAN
    :param vlan_name: Alphanumeric name of VLAN
    :param vlan_port_name: Alphanumeric Port name
    :param vlan_int_name: Alphanumeric name for the VLAN interface
    :param vlan_desc: Optional description to add to VLAN
    :param ipv4: Optional IPv4 address to assign to the interface.Defaults to nothing if not specified.
    :param vrf_name: VRF to attach the SVI to. Defaults to "default" if not specified
    :param vlan_port_desc: Optional description for the interface. Defaults to nothing if not specified.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        return _create_vlan_and_svi_v1(vlan_id, vlan_name, vlan_port_name, vlan_int_name, vlan_desc, ipv4,
                                vrf_name, vlan_port_desc, **kwargs)
    else:  # Updated else for when version is v10.04
        return _create_vlan_and_svi(vlan_id, vlan_name, vlan_port_name, vlan_int_name, vlan_desc, ipv4,
                             vrf_name, vlan_port_desc, **kwargs)


def _create_vlan_and_svi_v1(vlan_id, vlan_name, vlan_port_name, vlan_int_name, vlan_desc=None, ipv4=None,
                            vrf_name="default", vlan_port_desc=None, **kwargs):
    """
    Perform POST and PUT calls to create a new VLAN and SVI.

    :param vlan_id: Numeric ID of VLAN
    :param vlan_name: Alphanumeric name of VLAN
    :param vlan_port_name: Alphanumeric Port name
    :param vlan_int_name: Alphanumeric name for the VLAN interface
    :param vlan_desc: Optional description to add to VLAN
    :param ipv4: Optional IPv4 address to assign to the interface.Defaults to nothing if not specified.
    :param vrf_name: VRF to attach the SVI to. Defaults to "default" if not specified
    :param vlan_port_desc: Optional description for the interface. Defaults to nothing if not specified.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    # Add a new VLAN to the VLAN table
    success = create_vlan(vlan_id, vlan_name, vlan_desc, **kwargs)

    # Add a new entry to the Port table
    success = success and port.add_vlan_port(vlan_port_name, vlan_id, ipv4, vrf_name, vlan_port_desc, **kwargs)

    # Add a new entry to the Interface table
    return success and interface.add_vlan_interface(vlan_int_name, vlan_port_name, vlan_id, ipv4, vrf_name, vlan_port_desc, **kwargs)


def _create_vlan_and_svi(vlan_id, vlan_name, vlan_port_name, vlan_int_name, vlan_desc=None, ipv4=None,
                        vrf_name="default", vlan_port_desc=None, **kwargs):
    """
    Perform POST and PUT calls to create a new VLAN and SVI.

    :param vlan_id: Numeric ID of VLAN
    :param vlan_name: Alphanumeric name of VLAN
    :param vlan_port_name: Alphanumeric Port name
    :param vlan_int_name: Alphanumeric name for the VLAN interface
    :param vlan_desc: Optional description to add to VLAN
    :param ipv4: Optional IPv4 address to assign to the interface.Defaults to nothing if not specified.
    :param vrf_name: VRF to attach the SVI to. Defaults to "default" if not specified
    :param vlan_port_desc: Optional description for the interface. Defaults to nothing if not specified.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    # Add a new VLAN to the VLAN table
    success = create_vlan(vlan_id, vlan_name, vlan_desc, **kwargs)

    # Add a new entry to the Interface table
    return success and interface.add_vlan_interface(vlan_int_name, vlan_port_name, vlan_id, ipv4, vrf_name, vlan_port_desc, **kwargs)


def delete_vlan(vlan_id, **kwargs):
    """
    Perform a DELETE call to delete VLAN.

    :param vlan_id: Numeric ID of VLAN
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """

    if kwargs["url"].endswith("/v1/"):
        return _delete_vlan_v1(vlan_id, **kwargs)
    else:  # Updated else for when version is v10.04
        return _delete_vlan(vlan_id, **kwargs)


def _delete_vlan_v1(vlan_id, **kwargs):
    """
    Perform a DELETE call to delete VLAN.

    :param vlan_id: Numeric ID of VLAN
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """

    vlans_list = get_all_vlans(**kwargs)

    if "/rest/v1/system/vlans/%s" % vlan_id in vlans_list:

        target_url = kwargs["url"] + "system/vlans/%s" % vlan_id

        response = kwargs["s"].delete(target_url, verify=False)

        if not common_ops._response_ok(response, "DELETE"):
            logging.warning("FAIL: Deleting VLAN ID: '%s' failed with status code %d: %s"
                            % (vlan_id, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Deleting VLAN ID: '%s' succeeded" % vlan_id)
            return True
    else:
        logging.info("SUCCESS: No need to remove VLAN ID '%d' since it doesn't exist" % vlan_id)
        return True


def _delete_vlan(vlan_id, **kwargs):
    """
    Perform a DELETE call to delete VLAN.

    :param vlan_id: Numeric ID of VLAN
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """

    vlans_dict = get_all_vlans(**kwargs)

    if str(vlan_id) in vlans_dict:

        target_url = kwargs["url"] + "system/vlans/%s" % vlan_id

        response = kwargs["s"].delete(target_url, verify=False)

        if not common_ops._response_ok(response, "DELETE"):
            logging.warning("FAIL: Deleting VLAN ID: '%s' failed with status code %d: %s"
                            % (vlan_id, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Deleting VLAN ID: '%s' succeeded" % vlan_id)
            return True
    else:
        logging.info("SUCCESS: No need to remove VLAN ID '%d' since it doesn't exist" % vlan_id)
        return True


def delete_vlan_and_svi(vlan_id, vlan_port_name, **kwargs):
    """
    Perform PUT and DELETE calls to delete SVI and VLAN.

    :param vlan_id: Numeric ID of VLAN
    :param vlan_port_name: Name of SVI's entry in Port table
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    # Delete VLAN SVI
    success = interface.delete_interface(vlan_port_name, **kwargs)

    # Delete VLAN
    return success and delete_vlan(vlan_id, **kwargs)


def attach_vlan_acl(vlan_id, list_name, list_type, **kwargs):
    """
    Perform GET and PUT calls to attach an ACL to a VLAN

    :param vlan_id: Numeric ID of VLAN
    :param list_name: Alphanumeric name of the ACL
    :param list_type: Type should be one of "ipv4," "ipv6," or "mac"
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        return _attach_vlan_acl_v1(vlan_id, list_name, list_type, **kwargs)
    else:  # Updated else for when version is v10.04
        return _attach_vlan_acl(vlan_id, list_name, list_type, **kwargs)


def _attach_vlan_acl_v1(vlan_id, list_name, list_type, **kwargs):
    """
    Perform GET and PUT calls to attach an ACL to a VLAN

    :param vlan_id: Numeric ID of VLAN
    :param list_name: Alphanumeric name of the ACL
    :param list_type: Type should be one of "ipv4," "ipv6," or "mac"
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    vlan_data = get_vlan(vlan_id, depth=0, selector="configuration", **kwargs)

    if list_type == "ipv4":
        vlan_data['aclv4_in_cfg'] = "/rest/v1/system/acls/%s/%s" % (list_name, list_type)
        vlan_data['aclv4_in_cfg_version'] = random.randrange(9007199254740991)

    if list_type == "ipv6":
        vlan_data['aclv6_in_cfg'] = "/rest/v1/system/acls/%s/%s" % (list_name, list_type)
        vlan_data['aclv6_in_cfg_version'] = random.randrange(9007199254740991)

    if list_type == "mac":
        vlan_data['aclmac_in_cfg'] = "/rest/v1/system/acls/%s/%s" % (list_name, list_type)
        vlan_data['aclmac_in_cfg_version'] = random.randrange(9007199254740991)

    # must remove these fields from the data since they can't be modified
    vlan_data.pop('id', None)
    vlan_data.pop('type', None)

    target_url = kwargs["url"] + "system/vlans/%s" % vlan_id
    put_data = json.dumps(vlan_data, sort_keys=True, indent=4)

    response = kwargs["s"].put(target_url, data=put_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Updating %s ACL on VLAN %d to '%s' failed with status code %d: %s"
              % (list_type, vlan_id, list_name, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Updating %s ACL on VLAN %d to '%s' succeeded"
              % (list_type, vlan_id, list_name))
        return True


def _attach_vlan_acl(vlan_id, list_name, list_type, **kwargs):
    """
    Perform GET and PUT calls to attach an ACL to a VLAN

    :param vlan_id: Numeric ID of VLAN
    :param list_name: Alphanumeric name of the ACL
    :param list_type: Type should be one of "ipv4," "ipv6," or "mac"
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    vlan_data = get_vlan(vlan_id, depth=2, selector="writable", **kwargs)

    if list_type == "ipv4":
        vlan_data['aclv4_in_cfg'] = "/rest/v10.04/system/acls/%s,%s" % (list_name, list_type)
        vlan_data['aclv4_in_cfg_version'] = random.randrange(9007199254740991)

    if list_type == "ipv6":
        vlan_data['aclv6_in_cfg'] = "/rest/v10.04/system/acls/%s,%s" % (list_name, list_type)
        vlan_data['aclv6_in_cfg_version'] = random.randrange(9007199254740991)

    if list_type == "mac":
        vlan_data['aclmac_in_cfg'] = "/rest/v10.04/system/acls/%s,%s" % (list_name, list_type)
        vlan_data['aclmac_in_cfg_version'] = random.randrange(9007199254740991)

    # must remove these fields from the data since they can't be modified
    vlan_data.pop('id', None)
    vlan_data.pop('type', None)

    target_url = kwargs["url"] + "system/vlans/%s" % vlan_id
    put_data = json.dumps(vlan_data, sort_keys=True, indent=4)

    response = kwargs["s"].put(target_url, data=put_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Updating %s ACL on VLAN %d to '%s' failed with status code %d: %s"
              % (list_type, vlan_id, list_name, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Updating %s ACL on VLAN %d to '%s' succeeded"
              % (list_type, vlan_id, list_name))
        return True


def detach_vlan_acl(vlan_id, list_type, **kwargs):
    """
    Perform GET and PUT calls to detach ACL from a VLAN

    :param vlan_id: Numeric ID of VLAN
    :param list_type: Type should be one of "ipv4," "ipv6," or "mac"
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        return _detach_vlan_acl_v1(vlan_id, list_type, **kwargs)
    else:  # Updated else for when version is v10.04
        return _detach_vlan_acl(vlan_id, list_type, **kwargs)


def _detach_vlan_acl_v1(vlan_id, list_type, **kwargs):
    """
    Perform GET and PUT calls to detach ACL from a VLAN

    :param vlan_id: Numeric ID of VLAN
    :param list_type: Type should be one of "ipv4," "ipv6," or "mac"
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    vlan_data = get_vlan(vlan_id, depth=0, selector="configuration", **kwargs)

    if list_type == "ipv4":
        vlan_data.pop('aclv4_in_cfg', None)
        vlan_data.pop('aclv4_in_cfg_version', None)

    if list_type == "ipv6":
        vlan_data.pop('aclv6_in_cfg', None)
        vlan_data.pop('aclv6_in_cfg_version', None)

    if list_type == "mac":
        vlan_data.pop('aclmac_in_cfg', None)
        vlan_data.pop('aclmac_in_cfg_version', None)

    # must remove these fields from the data since they can't be modified
    vlan_data.pop('id', None)
    vlan_data.pop('type', None)

    target_url = kwargs["url"] + "system/vlans/%s" % vlan_id
    put_data = json.dumps(vlan_data, sort_keys=True, indent=4)

    response = kwargs["s"].put(target_url, data=put_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Removing %s ACL from VLAN %d failed with status code %d: %s"
              % (list_type, vlan_id, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Removing %s ACL from VLAN %d succeeded"
              % (list_type, vlan_id))
        return True


def _detach_vlan_acl(vlan_id, list_type, **kwargs):
    """
    Perform GET and PUT calls to detach ACL from a VLAN

    :param vlan_id: Numeric ID of VLAN
    :param list_type: Type should be one of "ipv4," "ipv6," or "mac"
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    vlan_data = get_vlan(vlan_id, depth=1, selector="writable", **kwargs)

    if list_type == "ipv4":
        vlan_data.pop('aclv4_in_cfg', None)
        vlan_data.pop('aclv4_in_cfg_version', None)

    if list_type == "ipv6":
        vlan_data.pop('aclv6_in_cfg', None)
        vlan_data.pop('aclv6_in_cfg_version', None)

    if list_type == "mac":
        vlan_data.pop('aclmac_in_cfg', None)
        vlan_data.pop('aclmac_in_cfg_version', None)

    target_url = kwargs["url"] + "system/vlans/%s" % vlan_id
    put_data = json.dumps(vlan_data, sort_keys=True, indent=4)

    response = kwargs["s"].put(target_url, data=put_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Removing %s ACL from VLAN %d failed with status code %d: %s"
              % (list_type, vlan_id, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Removing %s ACL from VLAN %d succeeded"
              % (list_type, vlan_id))
        return True


def port_set_vlan_mode(l2_port_name, vlan_mode, **kwargs):
    """
    Perform GET and PUT calls to set an L2 interface's VLAN mode (native-tagged, native-untagged, or access)

    :param l2_port_name: L2 interface's Port table entry name
    :param vlan_mode: A string, either 'native-tagged', 'native-untagged', or 'access', specifying the desired VLAN
        mode
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        return port._port_set_vlan_mode(l2_port_name, vlan_mode, **kwargs)
    else:  # Updated else for when version is v10.04
        return interface._port_set_vlan_mode(l2_port_name, vlan_mode, **kwargs)


def port_add_vlan_trunks(l2_port_name, vlan_trunk_ids=[], **kwargs):
    """
    Perform GET and PUT/POST calls to add specified VLANs to a trunk port. By default, this will also set the port to
    have 'no routing' and if there is not a native VLAN, will set the native VLAN to VLAN 1.

    :param l2_port_name: L2 interface's Port table entry name
    :param vlan_trunk_ids: List of VLANs to specify as allowed on the trunk port.  If empty, the interface will
        allow all VLANs on the trunk.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        return port._port_add_vlan_trunks(l2_port_name, vlan_trunk_ids, **kwargs)
    else:  # Updated else for when version is v10.04
        return interface._port_add_vlan_trunks(l2_port_name, vlan_trunk_ids, **kwargs)


def port_set_native_vlan(l2_port_name, vlan_id, tagged=True, **kwargs):
    """
    Perform GET and PUT/POST calls to set a VLAN to be the native VLAN on the trunk. Also gives the option to set
    the VLAN as tagged.

    :param l2_port_name: L2 interface's Port table entry name
    :param vlan_id: Numeric ID of VLAN to add to trunk port
    :param tagged: Boolean to determine if the native VLAN will be set as the tagged VLAN.  If False, the VLAN
        will be set as the native untagged VLAN
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        return port._port_set_native_vlan(l2_port_name, vlan_id, tagged, **kwargs)
    else:  # Updated else for when version is v10.04
        return interface._port_set_native_vlan(l2_port_name, vlan_id, tagged, **kwargs)


def port_delete_vlan_port(l2_port_name, vlan_id, **kwargs):
    """
    Perform GET and PUT calls to remove a VLAN from a trunk port

    :param l2_port_name: L2 interface's Port table entry name
    :param vlan_id: Numeric ID of VLAN to remove from trunk port
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        return port._delete_vlan_port(l2_port_name, vlan_id, **kwargs)
    else:  # Updated else for when version is v10.04
        return interface._delete_vlan_port(l2_port_name, vlan_id, **kwargs)


def port_set_untagged_vlan(l2_port_name, vlan_id, **kwargs):
    """
    Perform GET and PUT/POST calls to set a VLAN on an access port

    :param l2_port_name: L2 interface's Port table entry name
    :param vlan_id: Numeric ID of VLAN to set on access port
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        return port._port_set_untagged_vlan(l2_port_name, vlan_id, **kwargs)
    else:  # Updated else for when version is v10.04
        return interface._port_set_untagged_vlan(l2_port_name, vlan_id, **kwargs)


# (C) Copyright 2019-2020 Hewlett Packard Enterprise Development LP.
# Apache License 2.0

from pyaoscx import common_ops
from pyaoscx import port
from pyaoscx import interface

import json
import re
import logging


def create_l2_lag_interface(name, phys_ports, lacp_mode="passive", mc_lag=False, fallback_enabled=False,
                            vlan_ids_list=[], desc=None, admin_state="up", **kwargs):
    """
    Perform a POST call to create a Port table entry for L2 LAG interface.

    :param name: Alphanumeric name of LAG Port
    :param phys_ports: List of physical ports to aggregate (e.g. ["1/1/1", "1/1/2", "1/1/3"])
    :param lacp_mode: Should be either "passive" or "active." Defaults to "passive" if not specified.
    :param mc_lag: Boolean to determine if the LAG is multi-chassis. Defaults to False if not specified.
    :param fallback_enabled: Boolean to determine if the LAG uses LACP fallback. Defaults to False if not specified.
    :param vlan_ids_list: Optional list of integer VLAN IDs to add as trunk VLANS. Defaults to empty list if not specified.
    :param desc: Optional description for the interface. Defaults to nothing if not specified.
    :param admin_state: Optional administratively-configured state of the port.
        Defaults to "up" if not specified
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """

    if kwargs["url"].endswith("/v1/"):
        return _create_l2_lag_interface_v1(name, phys_ports, lacp_mode, mc_lag, fallback_enabled, vlan_ids_list, desc,
                                           admin_state, **kwargs)
    else:  # Updated else for when version is v10.04
        success = _create_l2_lag_interface(name, phys_ports, lacp_mode, mc_lag, fallback_enabled, vlan_ids_list, desc,
                                        admin_state, **kwargs)

        if mc_lag or fallback_enabled:
            return success and _update_l2_lag_interface(name, mc_lag, fallback_enabled, **kwargs)
        else:
            return success


def _create_l2_lag_interface_v1(name, phys_ports, lacp_mode="passive", mc_lag=False, fallback_enabled=False,
                                vlan_ids_list=[], desc=None, admin_state="up", **kwargs):
    """
    Perform a POST call to create a Port table entry for L2 LAG interface.

    :param name: Alphanumeric name of LAG Port
    :param phys_ports: List of physical ports to aggregate (e.g. ["1/1/1", "1/1/2", "1/1/3"])
    :param lacp_mode: Should be either "passive" or "active." Defaults to "passive" if not specified.
    :param mc_lag: Boolean to determine if the LAG is multi-chassis. Defaults to False if not specified.
    :param fallback_enabled: Boolean to determine if the LAG uses LACP fallback. Defaults to False if not specified.
    :param vlan_ids_list: Optional list of integer VLAN IDs to add as trunk VLANS. Defaults to empty list if not specified.
    :param desc: Optional description for the interface. Defaults to nothing if not specified.
    :param admin_state: Optional administratively-configured state of the port.
        Defaults to "up" if not specified
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    ports_list = port.get_all_ports(**kwargs)

    port_name_percents = common_ops._replace_special_characters(name)
    if "/rest/v1/system/ports/%s" % port_name_percents not in ports_list:

        # Extract LAG ID from LAG name
        lag_id = int(re.search('\d+', name).group())

        # For each port, add LAG ID to the Interface table entry, and delete the Port table entry
        for phys_port in phys_ports:
            interface.add_port_to_lag(phys_port, lag_id, **kwargs)

        interfaces = ["/rest/v1/system/interfaces/%s" % common_ops._replace_special_characters(phys_port)
                      for phys_port in phys_ports]
        port_data = {"admin": admin_state,
                     "interfaces": interfaces,
                     "name": name,
                     "routing": False,
                     "vlan_trunks": ["/rest/v1/system/vlans/%d" % vlan_id for vlan_id in vlan_ids_list],
                     "lacp": lacp_mode,
                     "other_config": {
                         "mclag_enabled": mc_lag,
                         "lacp-fallback": fallback_enabled
                        },
                     "vlan_mode": "native-untagged",
                     "vlan_tag": "/rest/v1/system/vlans/1"
                     }

        if desc is not None:
            port_data['description'] = desc

        target_url = kwargs["url"] + "system/ports"
        post_data = json.dumps(port_data, sort_keys=True, indent=4)

        response = kwargs["s"].post(target_url, data=post_data, verify=False)

        if not common_ops._response_ok(response, "POST"):
            logging.warning("FAIL: Adding Port table entry '%s' failed with status code %d: %s"
                  % (name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Adding Port table entry '%s' succeeded" % name)
            return True
    else:
        logging.info("SUCCESS: No need to add Port table entry '%s' because it already exists"
              % name)
        return True


def _create_l2_lag_interface(name, phys_ports, lacp_mode="passive", mc_lag=False, fallback_enabled=False,
                             vlan_ids_list=[], desc=None, admin_state="up", **kwargs):
    """
    Perform a POST call to create a Port table entry for L2 LAG interface.

    :param name: Alphanumeric name of LAG Port
    :param phys_ports: List of physical ports to aggregate (e.g. ["1/1/1", "1/1/2", "1/1/3"])
    :param lacp_mode: Should be either "passive" or "active." Defaults to "passive" if not specified.
    :param mc_lag: Boolean to determine if the LAG is multi-chassis. Defaults to False if not specified.
    :param fallback_enabled: Boolean to determine if the LAG uses LACP fallback. Defaults to False if not specified.
    :param vlan_ids_list: Optional list of integer VLAN IDs to add as trunk VLANS. Defaults to empty list if not specified.
    :param desc: Optional description for the interface. Defaults to nothing if not specified.
    :param admin_state: Optional administratively-configured state of the port.
        Defaults to "up" if not specified
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    ints_dict = interface.get_all_interfaces(**kwargs)

    if name not in ints_dict:

        # Extract LAG ID from LAG name
        lag_id = int(re.search('\d+', name).group())

        # For each port, add LAG ID to the Interface table entry
        for phys_port in phys_ports:
            interface.add_port_to_lag(phys_port, lag_id, **kwargs)

        interfaces = ["/rest/v10.04/system/interfaces/%s" % common_ops._replace_special_characters(phys_port)
                      for phys_port in phys_ports]
        int_data = {"admin": admin_state,
                    "interfaces": interfaces,
                    "name": name,
                    "type": "lag",
                    "routing": False,
                    "vlan_trunks": ["/rest/v10.04/system/vlans/%d" % vlan_id for vlan_id in vlan_ids_list],
                    "lacp": lacp_mode,
                    "other_config": {},
                    "vlan_mode": "native-untagged",
                    "vlan_tag": "/rest/v10.04/system/vlans/1"
                    }

        if desc is not None:
            int_data['description'] = desc

        target_url = kwargs["url"] + "system/interfaces"
        post_data = json.dumps(int_data, sort_keys=True, indent=4)

        response = kwargs["s"].post(target_url, data=post_data, verify=False)

        if not common_ops._response_ok(response, "POST"):
            logging.warning("FAIL: Adding Interface table entry '%s' failed with status code %d: %s"
                  % (name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Adding Interface table entry '%s' succeeded" % name)
            return True
    else:
        logging.info("SUCCESS: No need to add Interface table entry '%s' because it already exists"
              % name)
        return True


def _update_l2_lag_interface(name, mc_lag=False, fallback_enabled=False, **kwargs):
    """
    Perform GET and PUT calls to update the Interface table entry for L2 LAG interface.

    :param name: Alphanumeric name of LAG Port
    :param mc_lag: Boolean to determine if the LAG is multi-chassis. Defaults to False if not specified.
    :param fallback_enabled: Boolean to determine if the LAG uses LACP fallback. Defaults to False if not specified.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """

    int_name_percents = common_ops._replace_special_characters(name)

    int_data = interface.get_interface(name, 1, "writable", **kwargs)

    if int_data['interfaces']:
        int_data['interfaces'] = common_ops._dictionary_to_list_values(int_data['interfaces'])

    if int_data['vlan_trunks']:
        int_data['vlan_trunks'] = common_ops._dictionary_to_list_values(int_data['vlan_trunks'])

    if int_data['vlan_tag']:
        # Convert the dictionary to a URI string
        int_data['vlan_tag'] = common_ops._dictionary_to_string(int_data['vlan_tag'])

    int_data['other_config']['mclag_enabled'] = mc_lag
    int_data['other_config']['lacp-fallback'] = fallback_enabled

    target_url = kwargs["url"] + "system/interfaces/%s" % int_name_percents
    put_data = json.dumps(int_data, sort_keys=True, indent=4)

    response = kwargs["s"].put(target_url, data=put_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Updating LAG Interface entry '%s' "
                        "failed with status code %d: %s" % (name, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Updating LAG Interface entry '%s' "
                     "succeeded" % name)
        return True


def create_l3_lag_interface(name, phys_ports, ipv4, lacp_mode="passive", mc_lag=False, fallback_enabled=False,
                            desc=None, admin_state="up", vrf="default", **kwargs):
    """
    Perform a POST call to create a Port table entry for L3 LAG interface.

    :param name: Alphanumeric Port name
    :param phys_ports: List of physical ports to aggregate (e.g. ["1/1/1", "1/1/2", "1/1/3"])
    :param ipv4: IPv4 address to assign to the interface. Defaults to nothing if not specified.
    :param lacp_mode: Should be either "passive" or "active." Defaults to "passive" if not specified.
    :param mc_lag: Boolean to determine if the LAG is multi-chassis. Defaults to False if not specified.
    :param fallback_enabled: Boolean to determine if the LAG uses LACP fallback. Defaults to False if not specified.
    :param desc: Optional description for the interface. Defaults to nothing if not specified.
    :param admin_state: Optional administratively-configured state of the port.
        Defaults to "up" if not specified
    :param vrf: Name of the VRF to which the Port belongs. Defaults to "default" if not specified.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """

    if kwargs["url"].endswith("/v1/"):
        return _create_l3_lag_interface_v1(name, phys_ports, ipv4, lacp_mode, mc_lag, fallback_enabled,
                                           desc, admin_state, vrf, **kwargs)
    else:  # Updated else for when version is v10.04
        return _create_l3_lag_interface(name, phys_ports, ipv4, lacp_mode, mc_lag, fallback_enabled,
                                        desc, admin_state, vrf, **kwargs)


def create_l3_lag_interface(name, phys_ports, ipv4, lacp_mode="passive", mc_lag=False, fallback_enabled=False,
                            desc=None, admin_state="up", vrf="default", **kwargs):
    """
    Perform a POST call to create a Port table entry for L3 LAG interface.

    :param name: Alphanumeric Port name
    :param phys_ports: List of physical ports to aggregate (e.g. ["1/1/1", "1/1/2", "1/1/3"])
    :param ipv4: IPv4 address to assign to the interface. Defaults to nothing if not specified.
    :param lacp_mode: Should be either "passive" or "active." Defaults to "passive" if not specified.
    :param mc_lag: Boolean to determine if the LAG is multi-chassis. Defaults to False if not specified.
    :param fallback_enabled: Boolean to determine if the LAG uses LACP fallback. Defaults to False if not specified.
    :param desc: Optional description for the interface. Defaults to nothing if not specified.
    :param admin_state: Optional administratively-configured state of the port.
        Defaults to "up" if not specified
    :param vrf: Name of the VRF to which the Port belongs. Defaults to "default" if not specified.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """

    if kwargs["url"].endswith("/v1/"):
        return _create_l3_lag_interface_v1(name, phys_ports, ipv4, lacp_mode, mc_lag, fallback_enabled,
                                           desc, admin_state, vrf, **kwargs)
    else:  # Updated else for when version is v10.04
        return _create_l3_lag_interface(name, phys_ports, ipv4, lacp_mode, mc_lag, fallback_enabled,
                                        desc, admin_state, vrf, **kwargs)


def _create_l3_lag_interface_v1(name, phys_ports, ipv4, lacp_mode="passive", mc_lag=False, fallback_enabled=False,
                                desc=None, admin_state="up", vrf="default", **kwargs):
    """
    Perform a POST call to create a Port table entry for L3 LAG interface.

    :param name: Alphanumeric Port name
    :param phys_ports: List of physical ports to aggregate (e.g. ["1/1/1", "1/1/2", "1/1/3"])
    :param ipv4: IPv4 address to assign to the interface. Defaults to nothing if not specified.
    :param lacp_mode: Should be either "passive" or "active." Defaults to "passive" if not specified.
    :param mc_lag: Boolean to determine if the LAG is multi-chassis. Defaults to False if not specified.
    :param fallback_enabled: Boolean to determine if the LAG uses LACP fallback. Defaults to False if not specified.
    :param desc: Optional description for the interface. Defaults to nothing if not specified.
    :param admin_state: Optional administratively-configured state of the port.
        Defaults to "up" if not specified
    :param vrf: Name of the VRF to which the Port belongs. Defaults to "default" if not specified.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    ports_list = port.get_all_ports(**kwargs)

    port_name_percents = common_ops._replace_special_characters(name)
    if "/rest/v1/system/ports/%s" % port_name_percents not in ports_list:

        # Extract LAG ID from LAG name
        lag_id = int(re.search('\d+', name).group())

        # For each port, add LAG ID to the Interface table entry, and delete the Port table entry
        for phys_port in phys_ports:
            interface.add_port_to_lag(phys_port, lag_id, **kwargs)

        interfaces = ["/rest/v1/system/interfaces/%s" % common_ops._replace_special_characters(phys_port)
                      for phys_port in phys_ports]
        port_data = {"admin": admin_state,
                     "interfaces": interfaces,
                     "name": name,
                     "routing": True,
                     "vrf": "/rest/v1/system/vrfs/%s" % vrf,
                     "ip4_address": ipv4,
                     "lacp": lacp_mode,
                     "other_config": {
                        "mclag_enabled": mc_lag,
                        "lacp-fallback": fallback_enabled
                        },
                     }

        if desc is not None:
            port_data['description'] = desc

        if ipv4 is not None:
            port_data['ip4_address'] = ipv4

        target_url = kwargs["url"] + "system/ports"
        post_data = json.dumps(port_data, sort_keys=True, indent=4)

        response = kwargs["s"].post(target_url, data=post_data, verify=False)

        if not common_ops._response_ok(response, "POST"):
            logging.warning("FAIL: Adding Port table entry '%s' failed with status code %d: %s"
                  % (name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Adding Port table entry '%s' succeeded" % name)
            return True
    else:
        logging.info("SUCCESS: No need to add Port table entry '%s' because it already exists"
              % name)
        return True


def _create_l3_lag_interface(name, phys_ports, ipv4, lacp_mode="passive", mc_lag=False, fallback_enabled=False,
                             desc=None, admin_state="up", vrf="default", **kwargs):
    """
    Perform a POST call to create a Port table entry for L3 LAG interface.

    :param name: Alphanumeric Port name
    :param phys_ports: List of physical ports to aggregate (e.g. ["1/1/1", "1/1/2", "1/1/3"])
    :param ipv4: IPv4 address to assign to the interface. Defaults to nothing if not specified.
    :param lacp_mode: Should be either "passive" or "active." Defaults to "passive" if not specified.
    :param mc_lag: Boolean to determine if the LAG is multi-chassis. Defaults to False if not specified.
    :param fallback_enabled: Boolean to determine if the LAG uses LACP fallback. Defaults to False if not specified.
    :param desc: Optional description for the interface. Defaults to nothing if not specified.
    :param admin_state: Optional administratively-configured state of the port.
        Defaults to "up" if not specified
    :param vrf: Name of the VRF to which the Port belongs. Defaults to "default" if not specified.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """

    ints_dict = interface.get_all_interfaces(**kwargs)

    if name not in ints_dict:

        # Extract LAG ID from LAG name
        lag_id = int(re.search('\d+', name).group())

        # For each port, add LAG ID to the Interface table entry, and delete the Port table entry
        for phys_port in phys_ports:
            interface.add_port_to_lag(phys_port, lag_id, **kwargs)

        interfaces = ["/rest/v10.04/system/interfaces/%s" % common_ops._replace_special_characters(phys_port)
                      for phys_port in phys_ports]
        int_data = {"admin": admin_state,
                    "interfaces": interfaces,
                    "name": name,
                    "type": "lag",
                    "vrf": "/rest/v10.04/system/vrfs/%s" % vrf,
                    "routing": True,
                    "ip4_address": ipv4,
                    "lacp": lacp_mode,
                    # "other_config": {
                    #    "mclag_enabled": mc_lag,
                    #    "lacp-fallback": fallback_enabled
                    #    }

                    }
        """commented out the other_config since it causes error"""

        if desc is not None:
            int_data['description'] = desc

        target_url = kwargs["url"] + "system/interfaces"
        post_data = json.dumps(int_data, sort_keys=True, indent=4)

        response = kwargs["s"].post(target_url, data=post_data, verify=False)

        if not common_ops._response_ok(response, "POST"):
            logging.warning("FAIL: Adding Interface table entry '%s' failed with status code %d: %s"
                  % (name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Adding Interface table entry '%s' succeeded" % name)
            return True
    else:
        logging.info("SUCCESS: No need to add Interface table entry '%s' because it already exists"
              % name)
        return True


def delete_lag_interface(name, phys_ports, **kwargs):
    """
    Perform a DELETE call to delete a LAG interface. For v1, also remove the LAG ID from the port's Interface,
    and create the associated Port table entry.

    :param name: Alphanumeric name of LAG interface
    :param phys_ports: List of physical ports to aggregate (e.g. ["1/1/1", "1/1/2", "1/1/3"])
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        return _delete_lag_interface_v1(name, phys_ports, **kwargs)
    else:  # Updated else for when version is v10.04
        return _delete_lag_interface(name,  phys_ports, **kwargs)


def _delete_lag_interface_v1(name, phys_ports, **kwargs):
    """
    Perform a DELETE call to delete a LAG interface. Also, for each physical port, create the associated Port table
    entry, and remove the LAG ID from the Port and Interface entries by initializing them to default state.

    :param name: Alphanumeric name of LAG interface
    :param phys_ports: List of physical ports to aggregate (e.g. ["1/1/1", "1/1/2", "1/1/3"])
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    success = interface.delete_interface(name, **kwargs)

    # For each port, create a Port table entry, then initialize the Port and Interface entries to remove LAG
    for phys_port in phys_ports:
        success = success and interface.add_l2_interface(phys_port, **kwargs)
        success = success and port.initialize_port_entry(phys_port, **kwargs)

    return success


def _delete_lag_interface(name, phys_ports, **kwargs):
    """
    Perform a DELETE call to delete a LAG interface.  Also, for each physical port, remove the LAG ID from the Interface
    entries by initializing them to default state.

    :param name: Alphanumeric name of LAG interface
    :param phys_ports: List of physical ports to aggregate (e.g. ["1/1/1", "1/1/2", "1/1/3"])
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    success = interface.delete_interface(name, **kwargs)

    # For each port, initialize the Interface entry to remove LAG
    for phys_port in phys_ports:
        success = success and interface.initialize_interface_entry(phys_port, **kwargs)

    return success



# (C) Copyright 2019-2020 Hewlett Packard Enterprise Development LP.
# Apache License 2.0

from pyaoscx import common_ops

import logging


def get_all_mac_addrs(vlan_id, **kwargs):
    """
    Perform a GET call to get MAC address(es) for VLAN

    :param vlan_id: Numeric ID of VLAN
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: List of MAC address URIs
    """
    target_url = kwargs["url"] + "system/vlans/%d/macs" % vlan_id

    response = kwargs["s"].get(target_url, verify=False)
    mac_data = []

    if not common_ops._response_ok(response, "GET"):
        logging.warning("FAIL: Getting MAC address(es) of VLAN ID '%d' failed with status code %d: %s"
              % (vlan_id, response.status_code, response.text))
    else:
        logging.info("SUCCESS: Getting MAC address(es) of VLAN ID '%d' succeeded" % vlan_id)
        mac_data = response.json()
    return mac_data


def get_mac_info(vlan_id, mac_type, mac_addr, **kwargs):
    """
    Perform a GET call to get MAC info

    :param vlan_id: Numeric ID of VLAN
    :param mac_type: The source of the MAC address. Must be "dynamic," "VSX," "static," "VRRP,"
        "port-access-security," "evpn," or "hsc"
    :param mac_addr: MAC address
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Dictionary containing MAC data
    """
    target_url = kwargs["url"] + "system/vlans/%d/macs/%s/%s" % (vlan_id, mac_type, mac_addr)

    response = kwargs["s"].get(target_url, verify=False)
    mac_data = []

    if not common_ops._response_ok(response, "GET"):
        logging.warning("FAIL: Getting data for MAC '%s' of VLAN ID '%d' failed with status code %d: %s"
              % (mac_addr, vlan_id, response.status_code, response.text))
    else:
        logging.info("SUCCESS: Getting data for MAC '%s' of VLAN ID '%d' succeeded" % (mac_addr, vlan_id))
        mac_data = response.json()
    return mac_data


def get_all_mac_addresses_on_system(**kwargs):
    """
    Perform a GET call to get a list of all of the MAC address URIs for all VLANs on the system
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: List of all MAC address URIs on the system
    """
    target_url = kwargs["url"] + "system/vlans/*/macs"
    response = kwargs["s"].get(target_url, verify=False)

    mac_list = []
    if not common_ops._response_ok(response, "GET"):
        logging.warning("FAIL: Getting a list of all MAC addresses on the system, failed with status code %d: %s"
              % (response.status_code, response.text))
    else:
        logging.info("SUCCESS: Getting a list of all MAC addresses on the system succeeded")
        mac_list = response.json()
    return mac_list



# (C) Copyright 2019-2020 Hewlett Packard Enterprise Development LP.
# Apache License 2.0

from pyaoscx import common_ops

import logging


def get_system_info(params={}, **kwargs):
    """
    Perform a GET call to get system information

    :param params: Dictionary of optional parameters for the GET request
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Dictionary containing system information
    """
    target_url = kwargs["url"] + "system"

    response = kwargs["s"].get(target_url, params=params, verify=False)

    if not common_ops._response_ok(response, "GET"):
        logging.warning("FAIL: Getting dictionary of system information failed with status code %d: %s"
              % (response.status_code, response.text))
        system_info_dict = {}
    else:
        logging.info("SUCCESS: Getting dictionary of system information succeeded")
        system_info_dict = response.json()

    return system_info_dict


def get_chassis_info(params={}, **kwargs):
    """
    Perform a GET call to get the chassis information, such as product info, reboot statistics, selftest info, and more.

    :param params: Dictionary of optional parameters for the GET request
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Dictionary containing chassis information
    """

    if kwargs["url"].endswith("/v1/"):
        target_url = kwargs["url"] + "system/subsystems/chassis/1"
    else:
        # Else logic designed for v10.04 and later
        target_url = kwargs["url"] + "system/subsystems/chassis,1"

    response = kwargs["s"].get(target_url, params=params, verify=False)

    if not common_ops._response_ok(response, "GET"):
        logging.warning("FAIL: Getting dictionary of chassis information failed with status code %d: %s"
              % (response.status_code, response.text))
        chassis_info_dict = {}
    else:
        logging.info("SUCCESS: Getting dictionary of chassis information succeeded")
        chassis_info_dict = response.json()

    return chassis_info_dict


def get_product_info(params={}, **kwargs):
    """
    Perform a GET call to get the product information, such as MAC, Part Number, Model, and Serial

    :param params: Dictionary of optional parameters for the GET request
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Dictionary containing product information
    """

    if kwargs["url"].endswith("/v1/"):
        target_url = kwargs["url"] + "system/subsystems/chassis/1?attributes=product_info"
    else:
        # Else logic designed for v10.04 and later
        target_url = kwargs["url"] + "system/subsystems/chassis,1?attributes=product_info"

    response = kwargs["s"].get(target_url, params=params, verify=False)

    if not common_ops._response_ok(response, "GET"):
        logging.warning("FAIL: Getting dictionary of product information failed with status code %d: %s"
              % (response.status_code, response.text))
        product_info_dict = {}
    else:
        logging.info("SUCCESS: Getting dictionary of product information succeeded")
        product_info_dict = response.json()

    return product_info_dict



# (C) Copyright 2019-2020 Hewlett Packard Enterprise Development LP.
# Apache License 2.0

from pyaoscx import common_ops

import json
import logging


def get_dhcp_relay(vrf_name, port_name, **kwargs):
    """
    Perform a GET call to get DHCP data for an interface

    :param vrf_name: Alphanumeric name of VRF
    :param port_name: L3 interface's Port table entry name
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Dictionary containing DHCP Relay data for interface
    """
    if kwargs["url"].endswith("/v1/"):
        dhcp_relays = _get_dhcp_relay_v1(vrf_name, port_name, **kwargs)
    else:  # Updated else for when version is v10.04
        dhcp_relays = _get_dhcp_relay(vrf_name, port_name, **kwargs)
    return dhcp_relays


def _get_dhcp_relay_v1(vrf_name, port_name, **kwargs):
    """
    Perform a GET call to get DHCP data for an interface

    :param vrf_name: Alphanumeric name of VRF
    :param port_name: L3 interface's Port table entry name
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Dictionary containing DHCP Relay data for interface
    """
    payload = {"selector": "configuration"}

    target_url = kwargs["url"] + "system/dhcp_relays/%s/%s" % (vrf_name, port_name)
    response = kwargs["s"].get(target_url, verify=False, params=payload, timeout=2)

    if not common_ops._response_ok(response, "GET"):
        logging.warning("FAIL: Getting IPv4 DHCP helper(s) for Port '%s' failed with status code %d: %s"
              % (port_name, response.status_code, response.text))
    else:
        logging.info("SUCCESS: Getting IPv4 DHCP helper(s) for Port '%s' succeeded" % port_name)

    return response.json()


def _get_dhcp_relay(vrf_name, port_name, **kwargs):
    """
    Perform a GET call to get DHCP data for an interface

    :param vrf_name: Alphanumeric name of VRF
    :param port_name: L3 interface's Port table entry name
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Dictionary containing DHCP Relay data for interface
    """
    payload = {"selector": "writable"}

    target_url = kwargs["url"] + "system/dhcp_relays/%s,%s" % (vrf_name, port_name)
    response = kwargs["s"].get(target_url, verify=False, params=payload, timeout=2)

    if not common_ops._response_ok(response, "GET"):
        logging.warning("FAIL: Getting IPv4 DHCP helper(s) for Port '%s' failed with status code %d: %s"
              % (port_name, response.status_code, response.text))
    else:
        logging.info("SUCCESS: Getting IPv4 DHCP helper(s) for Port '%s' succeeded" % port_name)

    return response.json()


def get_all_dhcp_relays(**kwargs):
    """
    Perform a GET call to get a list (or dictionary) of all entries in DHCP Relays table

    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: List/dict of all DHCP helpers in the table
    """
    target_url = kwargs["url"] + "system/dhcp_relays"

    response = kwargs["s"].get(target_url, verify=False)

    if not common_ops._response_ok(response, "GET"):
        logging.warning("FAIL: Getting list/dict of all DHCP Relay table entries failed with status code %d: %s"
              % (response.status_code, response.text))
    else:
        logging.info("SUCCESS: Getting list/dict of all DHCP Relay table entries succeeded")

    dhcp_helpers = response.json()
    return dhcp_helpers


def add_dhcp_relays(port_name, vrf_name, ipv4_helper_addresses, **kwargs):
    """
    Perform a POST call to add IPv4 DHCP helper(s) for an L3 interface. If there are already IPv4 helpers, the new
    helpers are added in addition to the already existing helpers.

    :param port_name: Alphanumeric name of the Port
    :param vrf_name: Alphanumeric name of the VRF
    :param ipv4_helper_addresses: List of IPv4 addresses to add as DHCP helpers
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """

    if kwargs["url"].endswith("/v1/"):
        return _add_dhcp_relays_v1(port_name, vrf_name, ipv4_helper_addresses, **kwargs)
    else:  # Updated else for when version is v10.04
        return _add_dhcp_relays(port_name, vrf_name, ipv4_helper_addresses, **kwargs)


def _add_dhcp_relays_v1(port_name, vrf_name, ipv4_helper_addresses, **kwargs):
    """
    Perform a POST call to add IPv4 DHCP helper(s) for an L3 interface. If there are already IPv4 helpers, the new
    helpers are added in addition to the already existing helpers.

    :param port_name: Alphanumeric name of the Port
    :param vrf_name: Alphanumeric name of the VRF
    :param ipv4_helper_addresses: List of IPv4 addresses to add as DHCP helpers
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """

    dhcp_relays_list = get_all_dhcp_relays(**kwargs)

    if "/rest/v1/system/dhcp_relays/%s/%s" % (vrf_name, port_name) not in dhcp_relays_list:
        dhcp_relays = {
                    "port": kwargs["url"] + "system/ports/%s" % port_name,
                    "vrf": kwargs["url"] + "system/vrfs/%s" % vrf_name,
                    "ipv4_ucast_server": ipv4_helper_addresses
                    }

        target_url = kwargs["url"] + "system/dhcp_relays"
        post_data = json.dumps(dhcp_relays, sort_keys=True, indent=4)

        response = kwargs["s"].post(target_url, data=post_data, verify=False)

        if not common_ops._response_ok(response, "POST"):
            logging.warning("FAIL: Adding IPv4 DHCP helpers %s to SVI Port '%s' failed with status code %d: %s" %
                  (repr(ipv4_helper_addresses), port_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Adding IPv4 DHCP helpers '%s' to SVI Port '%s' succeeded" %
                  (repr(ipv4_helper_addresses), port_name))
            return True

    else:
        dhcp_data = get_dhcp_relay(vrf_name, port_name, **kwargs)

        dhcp_data['ipv4_ucast_server'] = common_ops._list_remove_duplicates(
            dhcp_data['ipv4_ucast_server'] + ipv4_helper_addresses)

        if len(dhcp_data['ipv4_ucast_server']) > 8:
            raise Exception("Can't have more than 8 IPv4 DHCP helpers per interface!")

        dhcp_data.pop('port', None)  # Must remove this item from json since it can't be modified
        dhcp_data.pop('vrf', None)  # Must remove this item from json since it can't be modified

        target_url = kwargs["url"] + "system/dhcp_relays/%s/%s" % (vrf_name, port_name)
        put_data = json.dumps(dhcp_data, sort_keys=True, indent=4)

        response = kwargs["s"].put(target_url, data=put_data, verify=False)

        if not common_ops._response_ok(response, "PUT"):
            logging.warning("FAIL: Adding IPv4 DHCP helpers %s to SVI Port '%s' failed with status code %d: %s" %
                  (repr(ipv4_helper_addresses), port_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Adding IPv4 DHCP helpers '%s' to SVI Port '%s' succeeded" %
                  (repr(ipv4_helper_addresses), port_name))
            return True


def _add_dhcp_relays(port_name, vrf_name, ipv4_helper_addresses, **kwargs):
    """
    Perform a POST call to add IPv4 DHCP helper(s) for an L3 interface. If there are already IPv4 helpers, the new
    helpers are added in addition to the already existing helpers.

    :param port_name: Alphanumeric name of the Port
    :param vrf_name: Alphanumeric name of the VRF
    :param ipv4_helper_addresses: List of IPv4 addresses to add as DHCP helpers
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """

    dhcp_relays_dict = get_all_dhcp_relays(**kwargs)

    if "%s,%s" % (vrf_name, port_name) not in dhcp_relays_dict:
        dhcp_relays = {
                    "port": "/rest/v10.04/system/interfaces/%s" % port_name,
                    "vrf": "/rest/v10.04/system/vrfs/%s" % vrf_name,
                    "ipv4_ucast_server": ipv4_helper_addresses
                    }

        target_url = kwargs["url"] + "system/dhcp_relays"
        post_data = json.dumps(dhcp_relays, sort_keys=True, indent=4)

        response = kwargs["s"].post(target_url, data=post_data, verify=False)

        if not common_ops._response_ok(response, "POST"):
            logging.warning("FAIL: Adding IPv4 DHCP helpers %s to SVI Port '%s' failed with status code %d: %s" %
                  (repr(ipv4_helper_addresses), port_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Adding IPv4 DHCP helpers '%s' to SVI Port '%s' succeeded" %
                  (repr(ipv4_helper_addresses), port_name))
            return True
    else:
        dhcp_data = get_dhcp_relay(vrf_name, port_name, **kwargs)

        dhcp_data['ipv4_ucast_server'] = common_ops._list_remove_duplicates(
            dhcp_data['ipv4_ucast_server'] + ipv4_helper_addresses)

        if len(dhcp_data['ipv4_ucast_server']) > 8:
            raise Exception("Can't have more than 8 IPv4 DHCP helpers per interface!")

        dhcp_data.pop('dhcp_relay_v6_mcast_servers', None)  # Must remove this item from json since it can't be modified

        target_url = kwargs["url"] + "system/dhcp_relays/%s,%s" % (vrf_name, port_name)
        put_data = json.dumps(dhcp_data, sort_keys=True, indent=4)

        response = kwargs["s"].put(target_url, data=put_data, verify=False)

        if not common_ops._response_ok(response, "PUT"):
            logging.warning("FAIL: Adding IPv4 DHCP helpers %s to SVI Port '%s' failed with status code %d: %s" %
                  (repr(ipv4_helper_addresses), port_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Adding IPv4 DHCP helpers '%s' to SVI Port '%s' succeeded" %
                  (repr(ipv4_helper_addresses), port_name))
            return True


def delete_dhcp_relays(port_name, vrf_name="default", **kwargs):
    """
    Perform a DELETE call to delete all the IPv4 DHCP helper(s) for an L3 interface.

    :param port_name: Alphanumeric name of the Port
    :param vrf_name: Alphanumeric name of the VRF
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """

    if kwargs["url"].endswith("/v1/"):
        return _delete_dhcp_relays_v1(port_name, vrf_name, **kwargs)
    else:  # Updated else for when version is v10.04
        return _delete_dhcp_relays(port_name, vrf_name, **kwargs)


def _delete_dhcp_relays_v1(port_name, vrf_name="default", **kwargs):
    """
    Perform a DELETE call to delete all the IPv4 DHCP helper(s) for an L3 interface.

    :param port_name: Alphanumeric name of the Port
    :param vrf_name: Alphanumeric name of the VRF
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """

    dhcp_helpers_list = get_all_dhcp_relays(**kwargs)

    if "/rest/v1/system/dhcp_relays/%s/%s" % (vrf_name, port_name) in dhcp_helpers_list:

        target_url = kwargs["url"] + "system/dhcp_relays/%s/%s" % (vrf_name, port_name)

        response = kwargs["s"].delete(target_url, verify=False)

        if not common_ops._response_ok(response, "DELETE"):
            logging.warning("FAIL: Deleting all DHCP relays from interface Port '%s' failed with status code %d: %s"
                  % (port_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Deleting all DHCP relays from interface Port '%s' succeeded" % port_name)
            return True
    else:
        logging.info("SUCCESS: No need to delete DHCP relays from SVI Port '%s' since they don't exist"
              % port_name)
        return True


def _delete_dhcp_relays(port_name, vrf_name="default", **kwargs):
    """
    Perform a DELETE call to delete all the IPv4 DHCP helper(s) for an L3 interface.

    :param port_name: Alphanumeric name of the Port
    :param vrf_name: Alphanumeric name of the VRF
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """

    dhcp_helpers_dict = get_all_dhcp_relays(**kwargs)

    if "%s,%s" % (vrf_name, port_name) in dhcp_helpers_dict:

        target_url = kwargs["url"] + "system/dhcp_relays/%s,%s" % (vrf_name, port_name)

        response = kwargs["s"].delete(target_url, verify=False)

        if not common_ops._response_ok(response, "DELETE"):
            logging.warning("FAIL: Deleting all DHCP relays from interface Port '%s' failed with status code %d: %s"
                  % (port_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Deleting all DHCP relays from interface Port '%s' succeeded" % port_name)
            return True
    else:
        logging.info("SUCCESS: No need to delete DHCP relays from SVI Port '%s' since they don't exist"
              % port_name)
        return True



# (C) Copyright 2019-2020 Hewlett Packard Enterprise Development LP.
# Apache License 2.0

from pyaoscx import common_ops, vrf

import json
import logging


def get_bgp_routers(vrf_name, **kwargs):
    """
    Perform a GET call to get a list of all BGP Router Autonomous System Number references

    :param vrf_name: Alphanumeric name of the VRF that we are retrieving all BGP ASNs from
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: List of all BGP Router ASNs in the table
    """
    target_url = kwargs["url"] + "system/vrfs/%s/bgp_routers" % vrf_name

    response = kwargs["s"].get(target_url, verify=False)

    if not common_ops._response_ok(response, "GET"):
        logging.warning("FAIL: Getting list of all BGP Router ASNs failed with status code %d: %s"
              % (response.status_code, response.text))
        bgp_list = {}
    else:
        logging.info("SUCCESS: Getting list of all BGP Router ASNs succeeded")
        bgp_list = response.json()

    return bgp_list


def create_bgp_asn(vrf_name, asn, router_id=None, **kwargs):
    """
    Perform a POST call to create a BGP Router Autonomous System Number

    :param vrf_name: Alphanumeric name of the VRF the BGP ASN belongs to
    :param asn: Integer that represents the Autonomous System Number
    :param router_id: Optional IPv4 address that functions as the BGP Router ID
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    bgp_data = {
      "asn": asn
    }

    if router_id:
        bgp_data['router_id'] = router_id

    target_url = kwargs["url"] + "system/vrfs/%s/bgp_routers" % vrf_name

    post_data = json.dumps(bgp_data, sort_keys=True, indent=4)
    response = kwargs["s"].post(target_url, data=post_data, verify=False, timeout=2)

    if not common_ops._response_ok(response, "POST"):
        logging.warning("FAIL: Creating BGP ASN '%s' on vrf %s failed with status code %d: %s"
              % (asn, vrf_name, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Creating BGP ASN '%s' succeeded on vrf %s" % (asn, vrf_name))
        return True


def delete_bgp_asn(vrf_name, asn, **kwargs):
    """
    Perform a DELETE call to remove a BGP Router Autonomous System Number

    :param vrf_name: Alphanumeric name of the VRF the BGP ASN belongs to
    :param asn: Integer that represents the Autonomous System Number
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    bgp_list = get_bgp_routers(vrf_name, **kwargs)

    if str(asn) in bgp_list:
        target_url = kwargs["url"] + "system/vrfs/%s/bgp_routers/%d" % (vrf_name, asn)
        response = kwargs["s"].delete(target_url, verify=False)

        if not common_ops._response_ok(response, "DELETE"):
            logging.warning("FAIL: Deleting BGP ASN '%s' on vrf %s failed with status code %d: %s"
                  % (asn, vrf_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Deleting BGP ASN '%s' succeeded on vrf %s" % (asn, vrf_name))
            return True
    else:
        logging.info("SUCCESS: No need to Delete BGP ASN '%s' as it does not exists!" % asn)
        return True


def get_bgp_neighbors_list(vrf_name, asn, **kwargs):
    """
    Perform a GET call to get a list of all BGP neighbors for the supplied Autonomous System Number

    :param vrf_name: Alphanumeric name of the VRF that we are retrieving all BGP ASNs from
    :param asn: Integer that represents the Autonomous System Number
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: List of all BGP neighbors in the table for the ASN
    """
    target_url = kwargs["url"] + "system/vrfs/%s/bgp_routers/%s/bgp_neighbors" % (vrf_name, asn)

    response = kwargs["s"].get(target_url, verify=False)

    if not common_ops._response_ok(response, "GET"):
        logging.warning("FAIL: Getting list of all BGP neighbors for ASN '%s' failed with status code %d: %s"
              % (asn, response.status_code, response.text))
        neighbor_list = {}
    else:
        logging.info("SUCCESS: Getting list of all BGP neighbors for ASN '%s' succeeded" % asn)
        neighbor_list = response.json()
    return neighbor_list


def create_bgp_neighbors(vrf_name, asn, group_ip, family_type="l2vpn_evpn", reflector=False, send_community=False,
                         local_interface="", **kwargs):
    """
    Perform a POST call to create BGP neighbors to the associated BGP ASN.  With l2vpn_evpn being True, this will
    also apply EVPN settings to the BGP neighbor configurations.
    Note that this functions has logic that works for both v1 and v10.04

    :param vrf_name: Alphanumeric name of the VRF the BGP ASN belongs to
    :param asn: Integer that represents the Autonomous System Number
    :param group_ip: IPv4 address or name of group of the neighbors that functions as the BGP Router link
    :param family_type: Alphanumeric to specify what type of neighbor settings to configure. The options are 'l2vpn-evpn',
        'ipv4-unicast', or 'ipv6-unicast'. When setting to l2vpn-evpn, the neighbor configurations also will add
        route-reflector-client and send-community settings.
    :param reflector: Boolean value to determine whether this neighbor has route reflector enabled.  Default is False.
    :param send_community: Boolean value to determine whether this neighbor has send-community enabled.  Default is False.
    :param local_interface: Optional alphanumeric to specify which interface the neighbor will apply to.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if family_type not in ['l2vpn-evpn', 'ipv4-unicast', 'ipv6-unicast']:
        raise Exception("ERROR: family_type should be 'l2vpn-evpn', 'ipv4-unicast', or 'ipv6-unicast'")

    neighbor_list = get_bgp_neighbors_list(vrf_name, asn, **kwargs)

    if group_ip not in neighbor_list:
        bgp_data = {
            "ip_or_group_name": group_ip,
            "is_peer_group": False,
            "remote_as": asn,
            "shutdown": False,
            "activate": {
                "ipv4-unicast": False,
                "ipv6-unicast": False,
                "l2vpn-evpn": False
            },
            "next_hop_unchanged": {
                "l2vpn-evpn": False
            },
            "route_reflector_client": {
                "ipv4-unicast": False,
                "ipv6-unicast": False,
                "l2vpn-evpn": False
            },
            "send_community": {
                "ipv4-unicast": "none",
                "ipv6-unicast": "none",
                "l2vpn-evpn": "none"
            }
        }

        if local_interface:
            int_percents = common_ops._replace_special_characters(local_interface)
            if kwargs["url"].endswith("/v1/"):
                bgp_data.update({'local_interface': "/rest/v1/system/ports/%s" % int_percents})
            else:
                # Else logic designed for v10.04 and later
                bgp_data.update({'local_interface': "/rest/v10.04/system/interfaces/%s" % int_percents})

        bgp_data['activate'][family_type] = True

        if send_community:
            bgp_data['send_community'][family_type] = "both"

        if reflector:
            bgp_data['route_reflector_client'][family_type] = reflector

        target_url = kwargs["url"] + "system/vrfs/%s/bgp_routers/%s/bgp_neighbors" % (vrf_name, asn)

        post_data = json.dumps(bgp_data, sort_keys=True, indent=4)

        response = kwargs["s"].post(target_url, data=post_data, verify=False, timeout=2)

        if not common_ops._response_ok(response, "POST"):
            logging.warning("FAIL: Creating BGP Neighbor for ASN '%s' on interface %s failed with status code %d: %s"
                  % (asn, local_interface, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Creating BGP Neighbor for ASN '%s' succeeded on interface %s" % (asn, local_interface))
            return True
    else:
        logging.info("SUCCESS: BGP Neighbor already exists for ASN '%s' on interface '%s'." % (asn, local_interface))
        return True


def create_bgp_vrf(vrf_name, asn, redistribute, **kwargs):
    """
    Perform a POST call to create BGP VRF settings for the associated BGP ASN.
    Note that this functions has logic that works for both v1 and v10.04

    :param vrf_name: Alphanumeric name of the VRF the BGP ASN belongs to
    :param asn: Integer that represents the Autonomous System Number
    :param redistribute: Optional alphanumeric to specify which types of routes that should be redistributed by BGP. The
        options are "ipv4-unicast" or "ipv6-unicast".
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if redistribute not in ['ipv4-unicast', 'ipv6-unicast']:
        raise Exception("ERROR: redistribute should be 'ipv4-unicast' or 'ipv6-unicast'")

    vrf_list = vrf.get_all_vrfs(**kwargs)

    if kwargs["url"].endswith("/v1/"):
        vrf_check = "/rest/v1/system/vrfs/%s" % vrf_name
    else:   # Updated else for when version is v10.04
        vrf_check = vrf_name

    if vrf_check in vrf_list:
        bgp_vrf_data = {
            "asn": asn
        }
        if redistribute == 'ipv4-unicast':
            bgp_vrf_data['redistribute'] = {
                "ipv4-unicast": ["connected"]
            }
        elif redistribute == 'ipv6-unicast':
            bgp_vrf_data['redistribute'] = {
                "ipv6-unicast": ["connected"]
            }

        target_url = kwargs["url"] + "system/vrfs/%s/bgp_routers" % vrf_name
        post_data = json.dumps(bgp_vrf_data, sort_keys=True, indent=4)

        response = kwargs["s"].post(target_url, data=post_data, verify=False, timeout=2)

        if not common_ops._response_ok(response, "POST"):
            logging.warning("FAIL: Creating BGP VRF '%s' for ASN '%s' failed with status code %d: %s"
                  % (vrf_name, asn, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Creating BGP VRF '%s' for ASN '%s' succeeded" % (vrf_name, asn))
            return True
    else:
        logging.warning("FAIL: Cannot create BGP VRF '%s' as VRF does not exist!" % vrf_name)
        return False


def delete_bgp_vrf(vrf_name, asn, **kwargs):
    """
    Perform a DELETE call to remove BGP VRF settings for the associated BGP ASN.
    Note that this functions has logic that works for both v1 and v10.04

    :param vrf_name: Alphanumeric name of the VRF the BGP ASN belongs to
    :param asn: Integer that represents the Autonomous System Number
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    vrf_list = vrf.get_all_vrfs(**kwargs)

    if kwargs["url"].endswith("/v1/"):
        vrf_check = "/rest/v1/system/vrfs/%s" % vrf_name
    else:   # Updated else for when version is v10.04
        vrf_check = vrf_name

    if vrf_check in vrf_list:
        target_url = kwargs["url"] + "system/vrfs/%s/bgp_routers" % vrf_name
        response = kwargs["s"].delete(target_url, verify=False)

        if not common_ops._response_ok(response, "DELETE"):
            logging.warning("FAIL: Deleting BGP VRF '%s' for ASN '%s' failed with status code %d: %s"
                  % (vrf_name, asn, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Deleting BGP VRF '%s' for ASN '%s' succeeded" % (vrf_name, asn))
            return True
    else:
        logging.info("SUCCESS: No need to Delete BGP VRF '%s' as VRF does not exists." % vrf_name)
        return True



# (C) Copyright 2019-2020 Hewlett Packard Enterprise Development LP.
# Apache License 2.0

from pyaoscx import common_ops

import json
import logging


def get_all_vrfs(**kwargs):
    """
    Perform a GET call to get a list (or dictionary) of all entries in VRF table

    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: List/dict of all VRFs in the table
    """
    target_url = kwargs["url"] + "system/vrfs"

    response = kwargs["s"].get(target_url, verify=False)

    if not common_ops._response_ok(response, "GET"):
        logging.warning("FAIL: Getting list/dict of all VRF table entries failed with status code %d: %s"
              % (response.status_code, response.text))
        vrfs = []
    else:
        logging.info("SUCCESS: Getting list/dict of all VRF table entries succeeded")
        vrfs = response.json()

    return vrfs


def add_vrf(vrf_name, route_distinguisher=None, vrf_type="user", **kwargs):
    """
    Perform a POST call to create a new VRF, and add a route distinguisher if desired.

    :param vrf_name: Alphanumeric name of VRF
    :param route_distinguisher: Optional route distinguisher to add. Defaults to nothing if not specified.
    :param vrf_type: Optional VRF type. Defaults to "user" if not specified.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        return add_vrf_v1(vrf_name, route_distinguisher, vrf_type, **kwargs)
    else:  # Updated else for when version is v10.04
        return _add_vrf(vrf_name, route_distinguisher, vrf_type, **kwargs)


def add_vrf_v1(vrf_name, route_distinguisher=None, vrf_type="user", **kwargs):
    """
    Perform a POST call to create a new VRF, and add a route distinguisher if desired.

    :param vrf_name: name of VRF
    :param route_distinguisher: Optional route distinguisher to add. Defaults to nothing if not specified.
    :param vrf_type: Optional VRF type. Defaults to "user" if not specified.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    vrfs_list = get_all_vrfs(**kwargs)

    if "/rest/v1/system/vrfs/%s" % vrf_name not in vrfs_list:
        vrf_data = {"name": vrf_name, "type": vrf_type}

        if route_distinguisher is not None:
            vrf_data["rd"] = route_distinguisher

        target_url = kwargs["url"] + "system/vrfs"
        post_data = json.dumps(vrf_data, sort_keys=True, indent=4)

        response = kwargs["s"].post(target_url, data=post_data, verify=False)

        if not common_ops._response_ok(response, "POST"):
            logging.warning("FAIL: Creating new VRF '%s' failed with status code %d: %s"
                            % (vrf_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Creating new VRF '%s' succeeded" % vrf_name)
            return True
    else:
        logging.info("SUCCESS: No need to create VRF '%s' since it already exists" % vrf_name)
        return True


def _add_vrf(vrf_name, route_distinguisher=None, vrf_type="user", **kwargs):
    """
    Perform a POST call to create a new VRF, and add a route distinguisher if desired.

    :param vrf_name: name of VRF
    :param route_distinguisher: Optional route distinguisher to add. Defaults to nothing if not specified.
    :param vrf_type: Optional VRF type. Defaults to "user" if not specified.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    vrfs_dict = get_all_vrfs(**kwargs)

    if vrf_name not in vrfs_dict:
        vrf_data = {"name": vrf_name, "type": vrf_type}

        if route_distinguisher is not None:
            vrf_data["rd"] = route_distinguisher

        target_url = kwargs["url"] + "system/vrfs"
        post_data = json.dumps(vrf_data, sort_keys=True, indent=4)

        response = kwargs["s"].post(target_url, data=post_data, verify=False)

        if not common_ops._response_ok(response, "POST"):
            logging.warning("FAIL: Creating new VRF '%s' failed with status code %d: %s"
                            % (vrf_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Creating new VRF '%s' succeeded" % vrf_name)
            return True
    else:
        logging.info("SUCCESS: No need to create VRF '%s' since it already exists" % vrf_name)
        return True


def get_vrf(vrf_name, depth=0, selector=None, **kwargs):
    """
    Perform a GET call to get data for a VRF table entry

    :param vrf_name: Alphanumeric name of the VRF
    :param depth: Integer deciding how many levels into the API JSON that references will be returned.
    :param selector: Alphanumeric option to select specific information to return.  The options are 'configuration',
        'status', 'statistics' or 'writable'.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Dictionary containing the VRF data
    """
    if kwargs["url"].endswith("/v1/"):
        return _get_vrf_v1(vrf_name, depth, selector, **kwargs)
    else:   # Updated else for when version is v10.04
        return _get_vrf(vrf_name, depth, selector, **kwargs)


def _get_vrf_v1(vrf_name, depth=0, selector=None, **kwargs):
    """
    Perform a GET call to get data for a VRF table entry

    :param vrf_name: Alphanumeric name of the VRF
    :param depth: Integer deciding how many levels into the API JSON that references will be returned.
    :param selector: Alphanumeric option to select specific information to return.  The options are 'configuration',
        'status', 'statistics' or 'writable'.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Dictionary containing the VRF data
    """
    if selector not in ['configuration', 'status', 'statistics', None]:
        raise Exception("ERROR: Selector should be 'configuration', 'status', or 'statistics'")

    target_url = kwargs["url"] + "system/vrfs/%s" % vrf_name
    payload = {
        "depth": depth,
        "selector": selector
    }
    response = kwargs["s"].get(target_url, verify=False, params=payload, timeout=2)

    if not common_ops._response_ok(response, "GET"):
        logging.warning("FAIL: Getting VRF table entry '%s' failed with status code %d: %s"
                        % (vrf_name, response.status_code, response.text))
        vrf = []
    else:
        logging.info("SUCCESS: Getting VRF table entry '%s' succeeded" % vrf_name)
        vrf = response.json()

    return vrf


def _get_vrf(vrf_name, depth=1, selector=None, **kwargs):
    """
    Perform a GET call to get data for a VRF table entry

    :param vrf_name: Alphanumeric name of the VRF
    :param depth: Integer deciding how many levels into the API JSON that references will be returned.
    :param selector: Alphanumeric option to select specific information to return.  The options are 'configuration',
        'status', 'statistics' or 'writable'.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Dictionary containing the VRF data
    """

    if selector not in ['configuration', 'status', 'statistics', 'writable', None]:
        raise Exception("ERROR: Selector should be 'configuration', 'status', 'statistics', or 'writable'")

    target_url = kwargs["url"] + "system/vrfs/%s" % vrf_name
    payload = {
        "depth": depth,
        "selector": selector
    }
    response = kwargs["s"].get(target_url, verify=False, params=payload, timeout=2)

    if not common_ops._response_ok(response, "GET"):
        logging.warning("FAIL: Getting VRF table entry '%s' failed with status code %d: %s"
                        % (vrf_name, response.status_code, response.text))
        vrf = []
    else:
        logging.info("SUCCESS: Getting VRF table entry '%s' succeeded" % vrf_name)
        vrf = response.json()

    return vrf


def delete_vrf(vrf_name, **kwargs):
    """
    Perform a DELETE call to delete a VRF.
    Note that this functions has logic that works for both v1 and v10.04

    :param vrf_name: Alphanumeric name of VRF
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    vrf_list = get_all_vrfs(**kwargs)

    if kwargs["url"].endswith("/v1/"):
        vrf_check = "/rest/v1/system/vrfs/%s" % vrf_name
    else:
        # Else logic designed for v10.04 and later
        vrf_check = vrf_name

    if vrf_check in vrf_list:
        target_url = kwargs["url"] + "system/vrfs/%s" % vrf_name
        response = kwargs["s"].delete(target_url, verify=False)

        if not common_ops._response_ok(response, "DELETE"):
            logging.warning("FAIL: Deleting VRF '%s' failed with status code %d: %s"
                            % (vrf_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Deleting VRF '%s' succeeded" % vrf_name)
            return True
    else:
        logging.info("SUCCESS: No need to delete VRF '%s' since it doesn't exist"
              % vrf_name)
        return True


def add_vrf_address_family(vrf_name, family_type="ipv4_unicast", export_target=[], import_targets=[], **kwargs):
    """
    Perform a POST call to create a new VRF, and add a route distinguisher if desired.
    Note that this functions has logic that works for both v1 and v10.04

    :param vrf_name: Alphanumeric name of VRF
    :param family_type: Alphanumeric type of the Address Family.  The options are 'ipv4_unicast' and 'ipv6_unicast'.
        The default value is set to 'ipv4_unicast'.
    :param export_target: Optional list of export route targets.
    :param import_targets: Optional list of import route targets
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    vrf_list = get_all_vrfs(**kwargs)

    if family_type == "ipv4-unicast":
        family_type = "ipv4_unicast"
    elif family_type == "ipv6-unicast":
        family_type = "ipv6_unicast"

    if family_type not in ['ipv4_unicast', 'ipv6_unicast']:
        raise Exception("ERROR: family_type should be 'ipv4_unicast', or 'ipv6_unicast'")

    if kwargs["url"].endswith("/v1/"):
        vrf_check = "/rest/v1/system/vrfs/%s" % vrf_name
    else:
        # Else logic designed for v10.04 and later
        vrf_check = vrf_name

    if vrf_check in vrf_list:
        address_family_data = {
            "address_family": family_type,
            "export_route_targets": export_target,
            "import_route_targets": import_targets,
            "route_map": {}
        }

        target_url = kwargs["url"] + "system/vrfs/%s/vrf_address_families" % vrf_name
        post_data = json.dumps(address_family_data, sort_keys=True, indent=4)

        response = kwargs["s"].post(target_url, data=post_data, verify=False)

        if not common_ops._response_ok(response, "POST"):
            logging.warning("FAIL: Creating '%s' Address Family on VRF '%s' failed with status code %d: %s"
                            % (family_type, vrf_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Creating '%s' Address Family on VRF '%s' succeeded" % (family_type, vrf_name))
            return True
    else:
        logging.warning("FAIL: Cannot add Address Family to VRF '%s' since the VRF has not been created yet" % vrf_name)
        return True


def delete_vrf_address_family(vrf_name, family_type="ipv4_unicast", **kwargs):
    """
    Perform a DELETE call to remove a VRF address family.
    Note that this functions has logic that works for both v1 and v10.04

    :param vrf_name: Alphanumeric name of VRF
    :param family_type: Alphanumeric type of the Address Family.  The options are 'ipv4_unicast' and 'ipv6_unicast'.
        The default value is set to 'ipv4_unicast'.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    vrf_list = get_all_vrfs(**kwargs)

    if family_type == "ipv4-unicast":
        family_type = "ipv4_unicast"
    elif family_type == "ipv6-unicast":
        family_type = "ipv6_unicast"

    if family_type not in ['ipv4_unicast', 'ipv6_unicast']:
        raise Exception("ERROR: family_type should be 'ipv4_unicast', or 'ipv6_unicast'")

    if kwargs["url"].endswith("/v1/"):
        vrf_check = "/rest/v1/system/vrfs/%s" % vrf_name
    else:
        # Else logic designed for v10.04 and later
        vrf_check = vrf_name

    if vrf_check in vrf_list:
        target_url = kwargs["url"] + "system/vrfs/%s/vrf_address_families/%s" % (vrf_name, family_type)
        response = kwargs["s"].delete(target_url, verify=False)

        if not common_ops._response_ok(response, "DELETE"):
            logging.warning("FAIL: Deleting '%s' Address Family on VRF '%s' failed with status code %d: %s"
                            % (family_type, vrf_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Deleting '%s' Address Family on VRF '%s' succeeded" % (family_type, vrf_name))
            return True
    else:
        logging.info("SUCCESS: No need to delete Address Family to VRF '%s' since it does not exist" % vrf_name)
        return True



# (C) Copyright 2019-2020 Hewlett Packard Enterprise Development LP.
# Apache License 2.0

from pyaoscx import common_ops, interface, port

import json
import random
import logging


def get_all_acls(**kwargs):
    """
    Perform a GET call to get a list of all ACLs

    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: List of all ACLs in the table
    """
    target_url = kwargs["url"] + "system/acls"

    response = kwargs["s"].get(target_url, verify=False)

    if not common_ops._response_ok(response, "GET"):
        logging.warning("FAIL: Getting list of all ACLs failed with status code %d: %s"
              % (response.status_code, response.text))
    else:
        logging.info("SUCCESS: Getting list of all ACLs succeeded")

    acls_list = response.json()
    return acls_list


def create_acl(list_name, list_type, **kwargs):
    """
    Perform a POST call to create an ACL with no entries

    :param list_name: Alphanumeric name of the ACL
    :param list_type: Type should be one of "ipv4," "ipv6," or "mac"
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        return _create_acl_v1(list_name, list_type, **kwargs)
    else:   # Updated else for when version is v10.04
        return _create_acl(list_name, list_type, **kwargs)


def _create_acl_v1(list_name, list_type, **kwargs):
    """
    Perform a POST call to create an ACL with no entries

    :param list_name: Alphanumeric name of the ACL
    :param list_type: Type should be one of "ipv4," "ipv6," or "mac"
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    acls_list = get_all_acls(**kwargs)

    # ACL doesn't exist; create it
    if "/rest/v1/system/acls/%s/%s" % (list_name, list_type) not in acls_list:

        acl_data = {
            "name": list_name,
            "list_type": list_type
        }

        target_url = kwargs["url"] + "system/acls"
        post_data = json.dumps(acl_data, sort_keys=True, indent=4)

        response = kwargs["s"].post(target_url, data=post_data, verify=False, timeout=2)

        if not common_ops._response_ok(response, "POST"):
            logging.warning("FAIL: Creating %s ACL '%s' failed with status code %d: %s"
                  % (list_type, list_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Creating %s ACL '%s' succeeded" % (list_type, list_name))
            return True
    else:
        logging.info("SUCCESS: No need to create %s ACL '%s' since it already exists"
              % (list_type, list_name))
        return True


def _create_acl(list_name, list_type, **kwargs):
    """
    Perform a POST call to create an ACL with no entries

    :param list_name: Alphanumeric name of the ACL
    :param list_type: Type should be one of "ipv4," "ipv6," or "mac"
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """

    acls_list = get_all_acls(**kwargs)

    acl_key = "{},{}".format(list_name, list_type)
    acl_value = "/rest/v10.04/system/acls/" + acl_key

    # ACL doesn't exist; create it
    if acl_value not in acls_list.values():
        acl_data = {
            "name": list_name,
            "list_type": list_type
        }

        target_url = kwargs["url"] + "system/acls"
        post_data = json.dumps(acl_data, sort_keys=True, indent=4)

        response = kwargs["s"].post(target_url, data=post_data, verify=False, timeout=2)

        if not common_ops._response_ok(response, "POST"):
            logging.warning("FAIL: Creating %s ACL '%s' failed with status code %d: %s"
                  % (list_type, list_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Creating %s ACL '%s' succeeded" % (list_type, list_name))
            return True
    else:
        logging.info("SUCCESS: No need to create %s ACL '%s' since it already exists"
              % (list_type, list_name))
        return True


def get_all_acl_entries(list_name, list_type, **kwargs):
    """
    Perform a GET call to get all entries of an ACL

    :param list_name: Alphanumeric name of the ACL
    :param list_type: Type should be one of "ipv4," "ipv6," or "mac"
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Dictionary containing ACL entry URIs
    """
    if kwargs["url"].endswith("/v1/"):
        acl_entries = _get_all_acl_entries_v1(list_name, list_type, **kwargs)
    else:   # Updated else for when version is v10.04
        acl_entries = _get_all_acl_entries(list_name, list_type, **kwargs)

    return acl_entries


def _get_all_acl_entries_v1(list_name, list_type, **kwargs):
    """
    Perform a GET call to get all entries of an ACL

    :param list_name: Alphanumeric name of the ACL
    :param list_type: Type should be one of "ipv4," "ipv6," or "mac"
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Dictionary containing ACL entry URIs
    """
    target_url = kwargs["url"] + "system/acls/%s/%s/cfg_aces" % (list_name, list_type)

    response = kwargs["s"].get(target_url, verify=False)

    if not common_ops._response_ok(response, "GET"):
        logging.warning("FAIL: Getting dictionary of URIS of entries in %s ACL '%s' failed with status code %d: %s"
              % (list_type, list_name, response.status_code, response.text))
        acl_entries = {}
    else:
        logging.info("SUCCESS: Getting dictionary of URIs of entries in %s ACL '%s' succeeded" % (list_type, list_name))
        acl_entries = response.json()

    # for some reason, this API returns a list when empty, and a dictionary when there is data
    # make this function always return a dictionary,
    if not acl_entries:
        return {}
    else:
        return acl_entries


def _get_all_acl_entries(list_name, list_type, **kwargs):
    """
    Perform a GET call to get all entries of an ACL

    :param list_name: Alphanumeric name of the ACL
    :param list_type: Type should be one of "ipv4," "ipv6," or "mac"
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Dictionary containing ACL entry URIs
    """

    target_url = kwargs["url"] + "system/acls/%s,%s/cfg_aces" % (list_name, list_type)

    response = kwargs["s"].get(target_url, verify=False)

    if not common_ops._response_ok(response, "GET"):
        logging.warning("FAIL: Getting dictionary of URIS of entries in %s ACL '%s' failed with status code %d: %s"
              % (list_type, list_name, response.status_code, response.text))
    else:
        logging.info("SUCCESS: Getting dictionary of URIs of entries in %s ACL '%s' succeeded" % (list_type, list_name))

    acl_entries = response.json()

    # for some reason, this API returns a list when empty, and a dictionary when there is data
    # make this function always return a dictionary,
    if not acl_entries:
        return {}
    else:
        return acl_entries


def create_acl_entry(list_name, list_type, sequence_num, action, count=None, ip_protocol=None, src_ip=None, dst_ip=None,
                     dst_l4_port_min=None, dst_l4_port_max=None, src_mac=None, dst_mac=None, ethertype=None, **kwargs):
    """
    Perform a POST call to create an ACL entry

    :param list_name: Alphanumeric name of the ACL
    :param list_type: Type should be one of "ipv4," "ipv6," or "mac"
    :param sequence_num: Integer number of the sequence
    :param action: Action should be either "permit" or "deny"
    :param count: Optional boolean flag that when true, will make entry increment hit count for matched packets
    :param ip_protocol: Optional integer IP protocol number
    :param src_ip: Optional source IP address
    :param dst_ip: Optional destination IP address
    :param dst_l4_port_min: Optional minimum L4 port number in range; used in conjunction with dst_l4_port_max.
    :param dst_l4_port_max: Optional maximum L4 port number in range; used in conjunction with dst_l4_port_min.
    :param src_mac: Optional source MAC address
    :param dst_mac: Optional destination MAC address
    :param ethertype: Optional integer EtherType number
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """

    if kwargs["url"].endswith("/v1/"):
        return _create_acl_entry_v1(list_name, list_type, sequence_num, action, count, ip_protocol, src_ip, dst_ip,
                     dst_l4_port_min, dst_l4_port_max, src_mac, dst_mac, ethertype, **kwargs)
    else:   # Updated else for when version is v10.04
        return _create_acl_entry(list_name, list_type, sequence_num, action, count, ip_protocol, src_ip, dst_ip,
                     dst_l4_port_min, dst_l4_port_max, src_mac, dst_mac, ethertype, **kwargs)


def _create_acl_entry_v1(list_name, list_type, sequence_num, action, count=None, ip_protocol=None, src_ip=None, dst_ip=None,
                     dst_l4_port_min=None, dst_l4_port_max=None, src_mac=None, dst_mac=None, ethertype=None, **kwargs):
    """
    Perform a POST call to create an ACL entry

    :param list_name: Alphanumeric name of the ACL
    :param list_type: Type should be one of "ipv4," "ipv6," or "mac"
    :param sequence_num: Integer number of the sequence
    :param action: Action should be either "permit" or "deny"
    :param count: Optional boolean flag that when true, will make entry increment hit count for matched packets
    :param ip_protocol: Optional integer IP protocol number
    :param src_ip: Optional source IP address
    :param dst_ip: Optional destination IP address
    :param dst_l4_port_min: Optional minimum L4 port number in range; used in conjunction with dst_l4_port_max.
    :param dst_l4_port_max: Optional maximum L4 port number in range; used in conjunction with dst_l4_port_min.
    :param src_mac: Optional source MAC address
    :param dst_mac: Optional destination MAC address
    :param ethertype: Optional integer EtherType number
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    acl_entries_dict = get_all_acl_entries(list_name, list_type, **kwargs)

    if "/rest/v1/system/acls/%s/%s/cfg_aces/%d" % (list_name, list_type, sequence_num) not in acl_entries_dict.values():
        acl_entry_data = {
            "sequence_number": sequence_num,
            "action": action
        }

        if count is not None:
            acl_entry_data["count"] = count

        if ip_protocol is not None:
            acl_entry_data["protocol"] = ip_protocol

        if src_ip is not None:
            acl_entry_data["src_ip"] = src_ip

        if dst_ip is not None:
            acl_entry_data["dst_ip"] = dst_ip

        if dst_l4_port_min is not None:
            acl_entry_data["dst_l4_port_min"] = dst_l4_port_min

        if dst_l4_port_max is not None:
            acl_entry_data["dst_l4_port_max"] = dst_l4_port_max

        if src_mac is not None:
            acl_entry_data["src_mac"] = src_mac

        if dst_mac is not None:
            acl_entry_data["dst_mac"] = dst_mac

        if ethertype is not None:
            acl_entry_data["ethertype"] = ethertype

        target_url = kwargs["url"] + "system/acls/%s/%s/cfg_aces" % (list_name, list_type)
        post_data = json.dumps(acl_entry_data, sort_keys=True, indent=4)

        response = kwargs["s"].post(target_url, data=post_data, verify=False, timeout=2)

        if not common_ops._response_ok(response, "POST"):
            logging.warning("FAIL: Creating entry %d for %s ACL '%s' failed with status code %d: %s"
                  % (sequence_num, list_type, list_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Creating entry %d for %s ACL '%s' succeeded" % (sequence_num, list_type, list_name))
            return True
    else:
        logging.info("SUCCESS: No need to create entry %d for %s ACL '%s' since it already exists"
              % (sequence_num, list_type, list_name))
        return True


def _create_acl_entry(list_name, list_type, sequence_num, action, count=None, ip_protocol=None, src_ip=None, dst_ip=None,
                     dst_l4_port_min=None, dst_l4_port_max=None, src_mac=None, dst_mac=None, ethertype=None, **kwargs):
    """
    Perform a POST call to create an ACL entry

    :param list_name: Alphanumeric name of the ACL
    :param list_type: Type should be one of "ipv4," "ipv6," or "mac"
    :param sequence_num: Integer number of the sequence
    :param action: Action should be either "permit" or "deny"
    :param count: Optional boolean flag that when true, will make entry increment hit count for matched packets
    :param ip_protocol: Optional integer IP protocol number
    :param src_ip: Optional source IP address
    :param dst_ip: Optional destination IP address
    :param dst_l4_port_min: Optional minimum L4 port number in range; used in conjunction with dst_l4_port_max.
    :param dst_l4_port_max: Optional maximum L4 port number in range; used in conjunction with dst_l4_port_min.
    :param src_mac: Optional source MAC address
    :param dst_mac: Optional destination MAC address
    :param ethertype: Optional integer EtherType number
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    acl_entries_dict = get_all_acl_entries(list_name, list_type, **kwargs)

    ace_key = "{},{}".format(list_name, list_type)
    ace_value = "/rest/v10.04/system/acls/%s/cfg_aces/%d" % (ace_key, sequence_num)

    if ace_value not in acl_entries_dict.values():
        acl_entry_data = {
            "sequence_number": sequence_num,
            "action": action,
        }

        if count is not None:
            acl_entry_data["count"] = count

        if ip_protocol is not None:
            acl_entry_data["protocol"] = ip_protocol

        if src_ip is not None:
            acl_entry_data["src_ip"] = src_ip

        if dst_ip is not None:
            acl_entry_data["dst_ip"] = dst_ip

        if dst_l4_port_min is not None:
            acl_entry_data["dst_l4_port_min"] = dst_l4_port_min

        if dst_l4_port_max is not None:
            acl_entry_data["dst_l4_port_max"] = dst_l4_port_max

        if src_mac is not None:
            acl_entry_data["src_mac"] = src_mac

        if dst_mac is not None:
            acl_entry_data["dst_mac"] = dst_mac

        if ethertype is not None:
            acl_entry_data["ethertype"] = ethertype

        target_url = kwargs["url"] + "system/acls/%s/cfg_aces" % ace_key
        post_data = json.dumps(acl_entry_data, sort_keys=True, indent=4)

        response = kwargs["s"].post(target_url, data=post_data, verify=False, timeout=2)

        if not common_ops._response_ok(response, "POST"):
            logging.warning("FAIL: Creating entry %d for %s ACL '%s' failed with status code %d: %s"
                  % (sequence_num, list_type, list_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Creating entry %d for %s ACL '%s' succeeded" % (sequence_num, list_type, list_name))
            return True
    else:
        logging.info("SUCCESS: No need to create entry %d for %s ACL '%s' since it already exists"
              % (sequence_num, list_type, list_name))
        return True


def get_acl(list_name, list_type, **kwargs):
    """
    Perform a GET call to get details of a particular ACL

    :param list_name: Alphanumeric name of the ACL
    :param list_type: Type should be one of "ipv4," "ipv6," or "mac"
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Dictionary containing data about a particular ACL
    """

    if kwargs["url"].endswith("/v1/"):
        acl = _get_acl_v1(list_name, list_type, **kwargs)
    else:   # Updated else for when version is v10.04
        acl = _get_acl(list_name, list_type, **kwargs)
    return acl


def _get_acl_v1(list_name, list_type, **kwargs):
    """
    Perform a GET call to get details of a particular ACL

    :param list_name: Alphanumeric name of the ACL
    :param list_type: Type should be one of "ipv4," "ipv6," or "mac"
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Dictionary containing data about a particular ACL
    """
    target_url = kwargs["url"] + "system/acls/%s/%s" % (list_name, list_type)

    payload = {"selector": "configuration"}

    response = kwargs["s"].get(target_url, params=payload, verify=False)

    if not common_ops._response_ok(response, "GET"):
        logging.warning("FAIL: Getting %s ACL '%s' failed with status code %d: %s"
              % (list_type, list_name, response.status_code, response.text))
    else:
        logging.info("SUCCESS: Getting %s ACL '%s' succeeded" % (list_type, list_name))

    acl = response.json()
    return acl


def _get_acl(list_name, list_type, **kwargs):
    """
    Perform a GET call to get details of a particular ACL

    :param list_name: Alphanumeric name of the ACL
    :param list_type: Type should be one of "ipv4," "ipv6," or "mac"
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: Dictionary containing data about a particular ACL
    """
    acl_key = "{},{}".format(list_name, list_type)
    target_url = kwargs["url"] + "system/acls/%s?depth=2&selector=writable" % acl_key

    response = kwargs["s"].get(target_url, verify=False)

    if not common_ops._response_ok(response, "GET"):
        logging.warning("FAIL: Getting %s ACL '%s' failed with status code %d: %s"
              % (list_type, list_name, response.status_code, response.text))
    else:
        logging.info("SUCCESS: Getting %s ACL '%s' succeeded" % (list_type, list_name))

    acl = response.json()
    return acl


def update_acl(list_name, list_type, **kwargs):
    """
    Perform a PUT call to version-up an ACL. This is required whenever entries of an ACL are changed
    in any way.

    :param list_name: Alphanumeric name of the ACL
    :param list_type: Type should be one of "ipv4," "ipv6," or "mac"
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        return _update_acl_v1(list_name, list_type, **kwargs)
    else:   # Updated else for when version is v10.04
        return _update_acl(list_name, list_type, **kwargs)


def _update_acl_v1(list_name, list_type, **kwargs):
    """
    Perform a PUT call to version-up an ACL. This is required whenever entries of an ACL are changed
    in any way.

    :param list_name: Alphanumeric name of the ACL
    :param list_type: Type should be one of "ipv4," "ipv6," or "mac"
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    acl_data = get_acl(list_name, list_type, **kwargs)

    # must remove these fields from the data since they can't be modified
    acl_data.pop('name', None)
    acl_data.pop('list_type', None)

    acl_data['cfg_version'] = random.randint(-9007199254740991, 9007199254740991)

    target_url = kwargs["url"] + "system/acls/%s/%s" % (list_name, list_type)
    put_data = json.dumps(acl_data, sort_keys=True, indent=4)

    response = kwargs["s"].put(target_url, data=put_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Updating %s ACL '%s' failed with status code %d: %s"
              % (list_type, list_name, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Updating %s ACL '%s' succeeded" % (list_type, list_name))
        return True


def _update_acl(list_name, list_type, **kwargs):
    """
    Perform a PUT call to version-up an ACL. This is required whenever entries of an ACL are changed
    in any way.

    :param list_name: Alphanumeric name of the ACL
    :param list_type: Type should be one of "ipv4," "ipv6," or "mac"
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """

    acl_data = get_acl(list_name, list_type, **kwargs)
    acl_key = "{},{}".format(list_name, list_type)

    acl_data['cfg_version'] = random.randint(-9007199254740991, 9007199254740991)
    target_url = kwargs["url"] + "system/acls/%s" % acl_key
    put_data = json.dumps(acl_data, sort_keys=True, indent=4)

    response = kwargs["s"].put(target_url, data=put_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Updating %s ACL '%s' failed with status code %d: %s"
              % (list_type, list_name, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Updating %s ACL '%s' succeeded" % (list_type, list_name))
        return True


def delete_acl(list_name, list_type, **kwargs):
    """
    Perform a DELETE call to delete an ACL

    :param list_name: Alphanumeric name of the ACL
    :param list_type: Type should be one of "ipv4," "ipv6," or "mac"
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        return _delete_acl_v1(list_name, list_type, **kwargs)
    else:   # Updated else for when version is v10.04
        return _delete_acl(list_name, list_type, **kwargs)


def _delete_acl_v1(list_name, list_type, **kwargs):
    """
    Perform a DELETE call to delete an ACL

    :param list_name: Alphanumeric name of the ACL
    :param list_type: Type should be one of "ipv4," "ipv6," or "mac"
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """

    acls_list = get_all_acls(**kwargs)

    if "/rest/v1/system/acls/%s/%s" % (list_name, list_type) in acls_list:

        target_url = kwargs["url"] + "system/acls/%s/%s" % (list_name, list_type)

        response = kwargs["s"].delete(target_url, verify=False)

        if not common_ops._response_ok(response, "DELETE"):
            logging.warning("FAIL: Deleting %s ACL '%s' failed with status code %d: %s"
                  % (list_type, list_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Deleting %s ACL '%s' succeeded" % (list_type, list_name))
            return True
    else:
        logging.info("SUCCESS: No need to delete %s ACL '%s' since it doesn't exist"
              % (list_type, list_name))
        return True


def _delete_acl(list_name, list_type, **kwargs):
    """
    Perform a DELETE call to delete an ACL

    :param list_name: Alphanumeric name of the ACL
    :param list_type: Type should be one of "ipv4," "ipv6," or "mac"
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """

    acls_list = get_all_acls(**kwargs)

    acl_key = "{},{}".format(list_name, list_type)
    acl_value = "/rest/v10.04/system/acls/" + acl_key

    # ACL exists; delete it
    if acl_value in acls_list.values():

        target_url = kwargs["url"] + "system/acls/%s,%s" % (list_name, list_type)

        response = kwargs["s"].delete(target_url, verify=False)

        if not common_ops._response_ok(response, "DELETE"):
            logging.warning("FAIL: Deleting %s ACL '%s' failed with status code %d: %s"
                  % (list_type, list_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Deleting %s ACL '%s' succeeded" % (list_type, list_name))
            return True
    else:
        logging.info("SUCCESS: No need to delete %s ACL '%s' since it doesn't exist"
              % (list_type, list_name))
        return True


def delete_acl_entry(list_name, list_type, sequence_num, **kwargs):
    """
    Perform a DELETE call to delete an ACL entry

    :param list_name: Alphanumeric name of the ACL
    :param list_type: Type should be one of "ipv4," "ipv6," or "mac"
    :param sequence_num: Integer ID for the entry.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        return _delete_acl_entry_v1(list_name, list_type, sequence_num, **kwargs)
    else:   # Updated else for when version is v10.04
        return _delete_acl_entry(list_name, list_type, sequence_num, **kwargs)


def _delete_acl_entry_v1(list_name, list_type, sequence_num, **kwargs):
    """
    Perform a DELETE call to delete an ACL entry

    :param list_name: Alphanumeric name of the ACL
    :param list_type: Type should be one of "ipv4," "ipv6," or "mac"
    :param sequence_num: Integer ID for the entry.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """

    acl_entries_dict = get_all_acl_entries(list_name, list_type, **kwargs)

    if "/rest/v1/system/acls/%s/%s/cfg_aces/%d" % (list_name, list_type, sequence_num) in acl_entries_dict.values():

        target_url = kwargs["url"] + "system/acls/%s/%s/cfg_aces/%d" % (list_name, list_type, sequence_num)

        response = kwargs["s"].delete(target_url, verify=False)

        if not common_ops._response_ok(response, "DELETE"):
            logging.warning("FAIL: Deleting entry %d in %s ACL '%s' failed with status code %d: %s"
                  % (sequence_num, list_type, list_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Deleting entry %d in %s ACL '%s' succeeded"
                  % (sequence_num, list_type, list_name))
            return True
    else:
        logging.info("SUCCESS: No need to delete entry %d in %s ACL '%s' since it doesn't exist"
              % (sequence_num, list_type, list_name))
        return True


def _delete_acl_entry(list_name, list_type, sequence_num, **kwargs):
    """
    Perform a DELETE call to delete an ACL entry

    :param list_name: Alphanumeric name of the ACL
    :param list_type: Type should be one of "ipv4," "ipv6," or "mac"
    :param sequence_num: Integer ID for the entry.
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """

    acl_entries_dict = get_all_acl_entries(list_name, list_type, **kwargs)

    ace_key = "{},{}".format(list_name, list_type)
    ace_value = "/rest/v10.04/system/acls/%s/cfg_aces/%d" % (ace_key, sequence_num)

    if ace_value in acl_entries_dict.values():
        target_url = kwargs["url"] + "system/acls/%s,%s/cfg_aces/%d" % (list_name, list_type, sequence_num)

        response = kwargs["s"].delete(target_url, verify=False)

        if not common_ops._response_ok(response, "DELETE"):
            logging.warning("FAIL: Deleting entry %d in %s ACL '%s' failed with status code %d: %s"
                  % (sequence_num, list_type, list_name, response.status_code, response.text))
            return False
        else:
            logging.info("SUCCESS: Deleting entry %d in %s ACL '%s' succeeded"
                  % (sequence_num, list_type, list_name))
            return True
    else:
        logging.info("SUCCESS: No need to delete entry %d in %s ACL '%s' since it doesn't exist"
              % (sequence_num, list_type, list_name))
        return True


def update_port_acl_in(interface_name, acl_name, list_type, **kwargs):
    """
    Perform GET and PUT calls to apply ACL on an interface. This function specifically applies an ACL
    to Ingress traffic of the interface

    :param interface_name: Alphanumeric String that is the name of the interface on which the ACL
        is applied to
    :param acl_name: Alphanumeric String that is the name of the ACL
    :param list_type: Alphanumeric String of ipv4 or ipv6 to specify the type of ACL
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """

    if kwargs["url"].endswith("/v1/"):
        return _update_port_acl_in_v1(interface_name, acl_name, list_type, **kwargs)
    else:   # Updated else for when version is v10.04
        return _update_port_acl_in(interface_name, acl_name, list_type, **kwargs)


def _update_port_acl_in_v1(interface_name, acl_name, list_type, **kwargs):
    """
    Perform GET and PUT calls to apply ACL on an interface. This function specifically applies an ACL
    to Ingress traffic of the interface

    :param interface_name: Alphanumeric String that is the name of the interface on which the ACL
        is applied to
    :param acl_name: Alphanumeric String that is the name of the ACL
    :param list_type: Alphanumeric String of ipv4 or ipv6 to specify the type of ACL
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    port_name_percents = common_ops._replace_special_characters(interface_name)

    port_data = port.get_port(port_name_percents, depth=0, selector="configuration", **kwargs)

    # must remove these fields from the data since they can't be modified
    port_data.pop('name', None)
    port_data.pop('origin', None)

    acl_url = "/rest/v1/system/acls/%s/%s" % (acl_name, list_type)

    if list_type is "ipv6":
        port_data['aclv6_in_cfg'] = acl_url
        port_data['aclv6_in_cfg_version'] = random.randint(-9007199254740991, 9007199254740991)
    elif list_type is "ipv4":
        port_data['aclv4_in_cfg'] = acl_url
        port_data['aclv4_in_cfg_version'] = random.randint(-9007199254740991, 9007199254740991)

    target_url = kwargs["url"] + "system/ports/%s" % port_name_percents
    put_data = json.dumps(port_data, sort_keys=True, indent=4)

    response = kwargs["s"].put(target_url, data=put_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Applying ACL '%s' to Ingress of Interface '%s' failed with status code %d: %s"
              % (acl_name, interface_name, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Applying ACL '%s' to Ingress of Interface '%s' succeeded"
              % (acl_name, interface_name))
        return True


def _update_port_acl_in(interface_name, acl_name, list_type, **kwargs):
    """
    Perform GET and PUT calls to apply ACL on an interface. This function specifically applies an ACL
    to Ingress traffic of the interface.  This function's minimum supported version is v10.04 and later

    :param interface_name: Alphanumeric name of the interface on which the ACL is applied to
    :param acl_name: Alphanumeric name of the ACL
    :param list_type: Alphanumeric String of ipv4 or ipv6 to specify the type of ACL
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    int_name_percents = common_ops._replace_special_characters(interface_name)
    int_data = interface.get_interface(int_name_percents, depth=1, selector="writable", **kwargs)

    acl_key = "{},{}".format(acl_name, list_type)
    acl_value = "/rest/v10.04/system/acls/" + acl_key

    if interface_name.startswith('lag'):
        if int_data['interfaces']:
            int_data['interfaces'] = common_ops._dictionary_to_list_values(int_data['interfaces'])

    if list_type is "ipv6":
        int_data['aclv6_in_cfg'] = acl_value
        int_data['aclv6_in_cfg_version'] = random.randint(-9007199254740991, 9007199254740991)
    elif list_type is "ipv4":
        int_data['aclv4_in_cfg'] = acl_value
        int_data['aclv4_in_cfg_version'] = random.randint(-9007199254740991, 9007199254740991)

    target_url = kwargs["url"] + "system/interfaces/%s" % int_name_percents
    put_data = json.dumps(int_data, sort_keys=True, indent=4)

    response = kwargs["s"].put(target_url, data=put_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Updating ACL %s on Ingress for Port '%s' failed with status code %d: %s"
              % (acl_name, interface_name, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Updating ACL %s on Ingress for Port '%s' succeeded"
              % (acl_name, interface_name))
        return True


def update_port_acl_out(interface_name, acl_name, list_type, **kwargs):
    """
    Perform GET and PUT calls to apply ACL on an L3 interface. This function specifically applies an ACL
    to Egress traffic of the interface, which must be a routing interface

    :param interface_name: Alphanumeric String that is the name of the interface on which the ACL
        is applied to
    :param acl_name: Alphanumeric String that is the name of the ACL
    :param list_type: Alphanumeric String of ipv4 or ipv6 to specify the type of ACL
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """

    if kwargs["url"].endswith("/v1/"):
        return _update_port_acl_out_v1(interface_name, acl_name, list_type, **kwargs)
    else:   # Updated else for when version is v10.04
        return _update_port_acl_out(interface_name, acl_name, list_type, **kwargs)


def _update_port_acl_out_v1(interface_name, acl_name, list_type, **kwargs):
    """
    Perform GET and PUT calls to apply ACL on an L3 interface. This function specifically applies an ACL
    to Egress traffic of the interface, which must be a routing interface.  This function will set the interface
    to enable routing.

    :param interface_name: Alphanumeric String that is the name of the interface on which the ACL
        is applied to
    :param acl_name: Alphanumeric String that is the name of the ACL
    :param list_type: Alphanumeric String of ipv4 or ipv6 to specify the type of ACL
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    port_name_percents = common_ops._replace_special_characters(interface_name)

    port_data = port.get_port(port_name_percents, depth=0, selector="configuration", **kwargs)

    # must remove these fields from the data since they can't be modified
    port_data.pop('name', None)
    port_data.pop('origin', None)

    acl_url = "/rest/v1/system/acls/%s/%s" % (acl_name, list_type)

    if list_type is "ipv6":
        port_data['aclv6_out_cfg'] = acl_url
        port_data['aclv6_out_cfg_version'] = random.randint(-9007199254740991, 9007199254740991)
    elif list_type is "ipv4":
        port_data['aclv4_out_cfg'] = acl_url
        port_data['aclv4_out_cfg_version'] = random.randint(-9007199254740991, 9007199254740991)

    port_data['routing'] = True

    target_url = kwargs["url"] + "system/ports/%s" % port_name_percents
    put_data = json.dumps(port_data, sort_keys=True, indent=4)

    response = kwargs["s"].put(target_url, data=put_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Applying ACL '%s' to Egress on Interface '%s' failed with status code %d: %s"
              % (acl_name, interface_name, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Applying ACL '%s' to Egress on Interface '%s' succeeded"
              % (acl_name, interface_name))
        return True


def _update_port_acl_out(interface_name, acl_name, list_type, **kwargs):
    """
    Perform GET and PUT calls to apply ACL on an interface. This function specifically applies an ACL
    to Egress traffic of the interface, which must be a routing interface.  This function will set the interface
    to enable routing.

    :param interface_name: Alphanumeric name of the interface on which the ACL is applied to
    :param acl_name: Alphanumeric name of the ACL
    :param list_type: Alphanumeric String of ipv4 or ipv6 to specify the type of ACL
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    port_name_percents = common_ops._replace_special_characters(interface_name)
    int_data = interface.get_interface(port_name_percents, depth=1, selector="writable", **kwargs)

    acl_key = "{},{}".format(acl_name, list_type)
    acl_value = "/rest/v10.04/system/acls/" + acl_key

    if interface_name.startswith('lag'):
        if int_data['interfaces']:
            int_data['interfaces'] = common_ops._dictionary_to_list_values(int_data['interfaces'])

    if list_type is "ipv6":
        int_data['aclv6_out_cfg'] = acl_value
        int_data['aclv6_out_cfg_version'] = random.randint(-9007199254740991, 9007199254740991)
    elif list_type is "ipv4":
        int_data['aclv4_out_cfg'] = acl_value
        int_data['aclv4_out_cfg_version'] = random.randint(-9007199254740991, 9007199254740991)

    int_data['routing'] = True

    target_url = kwargs["url"] + "system/interfaces/%s" % port_name_percents
    put_data = json.dumps(int_data, sort_keys=True, indent=4)

    response = kwargs["s"].put(target_url, data=put_data, verify=False)

    if not common_ops._response_ok(response, "PUT"):
        logging.warning("FAIL: Applying ACL '%s' to Egress on Interface '%s' failed with status code %d: %s"
              % (acl_name, interface_name, response.status_code, response.text))
        return False
    else:
        logging.info("SUCCESS: Applying ACL '%s' to Egress on Interface '%s' succeeded"
              % (acl_name, interface_name))
        return True


def clear_interface_acl(interface_name, acl_type, **kwargs):
    """
    Perform GET and PUT calls to clear an interface's ACL

    :param interface_name: Alphanumeric name of the interface
    :param acl_type: Type of ACL: options are 'aclv4_out', 'aclv4_in', 'aclv6_in', or 'aclv6_out'
    :param kwargs:
        keyword s: requests.session object with loaded cookie jar
        keyword url: URL in main() function
    :return: True if successful, False otherwise
    """
    if kwargs["url"].endswith("/v1/"):
        return port._clear_port_acl(interface_name, acl_type, **kwargs)
    else:   # Updated else for when version is v10.04
        return interface._clear_interface_acl(interface_name, acl_type, **kwargs)
